<?php
/* Copyright (C) 2023-2024  Open-Dsi        <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

/**
 *   \file       htdocs/product/note.php
 *   \brief      Tab for notes on products
 *   \ingroup    societe
 */

// Load Dolibarr environment
$res=0;
// Try main.inc.php into web root known defined into CONTEXT_DOCUMENT_ROOT (not always defined)
if (! $res && ! empty($_SERVER["CONTEXT_DOCUMENT_ROOT"])) $res=@include($_SERVER["CONTEXT_DOCUMENT_ROOT"]."/main.inc.php");
// Try main.inc.php into web root detected using web root caluclated from SCRIPT_FILENAME
$tmp=empty($_SERVER['SCRIPT_FILENAME'])?'':$_SERVER['SCRIPT_FILENAME'];$tmp2=realpath(__FILE__); $i=strlen($tmp)-1; $j=strlen($tmp2)-1;
while($i > 0 && $j > 0 && isset($tmp[$i]) && isset($tmp2[$j]) && $tmp[$i]==$tmp2[$j]) { $i--; $j--; }
if (! $res && $i > 0 && file_exists(substr($tmp, 0, ($i+1))."/main.inc.php")) $res=@include(substr($tmp, 0, ($i+1))."/main.inc.php");
if (! $res && $i > 0 && file_exists(dirname(substr($tmp, 0, ($i+1)))."/main.inc.php")) $res=@include(dirname(substr($tmp, 0, ($i+1)))."/main.inc.php");
// Try main.inc.php using relative path
if (! $res && file_exists("../main.inc.php")) $res=@include("../main.inc.php");
if (! $res && file_exists("../../main.inc.php")) $res=@include("../../main.inc.php");
if (! $res && file_exists("../../../main.inc.php")) $res=@include("../../../main.inc.php");
if (! $res) die("Include of main fails");

require_once DOL_DOCUMENT_ROOT.'/core/lib/product.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/class/html.formaccounting.class.php';
require_once DOL_DOCUMENT_ROOT.'/product/class/product.class.php';
dol_include_once('/extendedproductaccountancy/class/productaccountancycode.class.php');

// Load translation files required by the page
$langs->loadLangs(array('products', 'accountancy', 'other', 'extendedproductaccountancy@extendedproductaccountancy'));

$id = GETPOST('id', 'int');
$ref = GETPOST('ref', 'alpha');
$action = GETPOST('action', 'aZ09');
$cancel = GETPOST('cancel', 'alpha');
$confirm = GETPOST('confirm', 'aZ09');
$lineid = GETPOST('lineid', 'int');
$backtopage = GETPOST('backtopage', 'alpha');

$object = new Product($db);
if ($id > 0 || !empty($ref)) {
	$result = $object->fetch($id, $ref);
	if ($result < 0) {
		setEventMessage($object->errorsToString(), 'errors');
	}
}
$is_product_card = $object->id > 0;

$productaccountancycode = new ProductAccountancyCode($db);
if ($lineid > 0) {
	$result =  $productaccountancycode->fetch($lineid);
	if ($result < 0) {
		setEventMessage($productaccountancycode->errorsToString(), 'errors');
	}
}

$search_product = GETPOST("search_product", 'alphanohtml');
$search_product_type = GETPOST("search_product_type", 'int');
$search_free_line = GETPOST("search_free_line", 'int');
$search_vatrate = GETPOST("search_vatrate", 'alphanohtml');
$search_key_values = $productaccountancycode->getSearchKeyValuesFromPost();
$optioncss = GETPOST('optioncss', 'alpha');

$limit = GETPOST('limit', 'int') ?GETPOST('limit', 'int') : $conf->liste_limit;
$sortfield = GETPOST("sortfield", 'alphanohtml');
$sortorder = GETPOST("sortorder", 'alphanohtml');
$page = GETPOSTISSET('pageplusone') ? (GETPOST('pageplusone') - 1) : GETPOST("page", 'int');
if (empty($page) || $page < 0 || GETPOST('button_search', 'alpha') || GETPOST('button_removefilter', 'alpha')) {
	$page = 0;
}     // If $page is not defined, or '' or -1 or if we click on clear filters or if we select empty mass action
$offset = $limit * $page;
$pageprev = $page - 1;
$pagenext = $page + 1;
if (!$sortfield) {
	$sortfield = (!$is_product_card ? "p.ref," : "") . "pac.tva_tx";
}
if (!$sortorder) {
	$sortorder = (!$is_product_card ? "ASC," : "") . "ASC";
}

if ($is_product_card) {
	$usercanread = (($object->type == Product::TYPE_PRODUCT && $user->hasRight('produit', 'lire')) || ($object->type == Product::TYPE_SERVICE && $user->hasRight('service', 'lire')));
	$usercancreate = (($object->type == Product::TYPE_PRODUCT && $user->hasRight('produit', 'creer')) || ($object->type == Product::TYPE_SERVICE && $user->hasRight('service', 'creer')));
	$usercandelete = (($object->type == Product::TYPE_PRODUCT && $user->hasRight('produit', 'supprimer')) || ($object->type == Product::TYPE_SERVICE && $user->hasRight('service', 'supprimer')));
	$usercanreadonlyone = false;
	$usercancreateonlyone = false;
	$usercandeleteonlyone = false;
} else {
	$usercanreadproduct = $user->hasRight('produit', 'lire');
	$usercancreateproduct = $user->hasRight('produit', 'creer');
	$usercandeleteproduct = $user->hasRight('produit', 'supprimer');
	$usercanreadservice = $user->hasRight('service', 'lire');
	$usercancreateservice = $user->hasRight('service', 'creer');
	$usercandeleteservice = $user->hasRight('service', 'supprimer');
	$usercanread = $usercanreadproduct || $usercanreadservice;
	$usercancreate = $usercancreateproduct || $usercancreateservice;
	$usercandelete = $usercandeleteproduct || $usercandeleteservice;
	$usercanreadonlyone = ($usercanreadproduct && !$usercanreadservice) || (!$usercanreadproduct && $usercanreadservice);
	$usercancreateonlyone = ($usercancreateproduct && !$usercancreateservice) || (!$usercancreateproduct && $usercancreateservice);
	$usercandeleteonlyone = ($usercandeleteproduct && !$usercandeleteservice) || (!$usercandeleteproduct && $usercandeleteservice);
}

// Security check
if ($user->socid) {
	$socid = $user->socid;
}

if ($is_product_card) {
	if ($object->type == $object::TYPE_PRODUCT) {
		restrictedArea($user, 'produit', $object->id, 'product&product', '', '');
	}
	if ($object->type == $object::TYPE_SERVICE) {
		restrictedArea($user, 'service', $object->id, 'product&product', '', '');
	}
}
if (empty($user->rights->accounting->chartofaccount)) {
	accessforbidden();
}
if (!empty($conf->global->PRODUCT_DISABLE_ACCOUNTING)) {
	accessforbidden('Accounting disabled');
}

$isInEEC = $mysoc->isInEEC();
$key_conf = $productaccountancycode->getKeyConf();
$global_product_types = array();
if (empty($usercancreateonlyone)) $global_product_types[10] = $langs->trans('All');
if (empty($usercancreateonlyone) || !empty($usercancreateproduct)) $global_product_types[Product::TYPE_PRODUCT] = $langs->trans('Product');
if (empty($usercancreateonlyone) || !empty($usercancreateservice)) $global_product_types[Product::TYPE_SERVICE] = $langs->trans('Service');

// Initialize technical object to manage hooks of page. Note that conf->hooks_modules contains array of hook context
$hookmanager->initHooks(array('productaccountancy'));

/*
 * Actions
 */

if ($cancel) {
	$action = '';
}

$parameters = array('id'=>$id, 'ref'=>$ref);
$reshook = $hookmanager->executeHooks('doActions', $parameters, $object, $action); // Note that $action and $object may have been modified by some hooks
if ($reshook < 0) {
	setEventMessages($hookmanager->error, $hookmanager->errors, 'errors');
}

if (empty($reshook)) {
	// Do we click on purge search criteria ?
	if (GETPOST('button_removefilter_x', 'alpha') || GETPOST('button_removefilter.x', 'alpha') || GETPOST('button_removefilter', 'alpha')) { // All tests are required to be compatible with all browsers
		$search_product = '';
		$search_product_type = -1;
		$search_free_line = -1;
		$search_vatrate = '';
		$search_key_values = array();
	}
	if ($search_product_type === "") $search_product_type = -1;
	if ($search_free_line === "") $search_free_line = -1;

	if ($action == 'addline' && $usercancreate) {
		// Add a product accountancy code
		$error = 0;

		$product_id = $is_product_card ? $object->id : GETPOST('product_id', 'int');
		$product_type = $product_id > 0 ? -1 : GETPOST('product_type', 'int');
		$free_line = $product_id > 0 ? 0 : GETPOST('free_line', 'int');

		if ($usercancreateonlyone) {
			// Test product type
			if ($product_id > 0) {
				$product_tmp = new Product($db);
				$result = $product_tmp->fetch($product_id);
				if ($result < 0) {
					setEventMessages($product_tmp->error, $product_tmp->errors, 'errors');
					$error++;
				} elseif (($product_tmp->type == Product::TYPE_PRODUCT && !$usercancreateproduct) || ($product_tmp->type == Product::TYPE_SERVICE && !$usercancreateservice)) {
					setEventMessage($langs->trans('ErrorEPAProductSelectedIsNotAuthorized'), 'errors');
					$error++;
				}
			} elseif (($product_type == Product::TYPE_PRODUCT && !$usercancreateproduct) || ($product_type == Product::TYPE_SERVICE && !$usercancreateservice)) {
				setEventMessage($langs->trans('ErrorEPAProductTypeSelectedIsNotAuthorized'), 'errors');
				$error++;
			}
		}

		$tva_tx_txt = GETPOST('tva_tx', 'alpha'); // tva_tx can be '8.5'  or  '8.5*'  or  '8.5 (XXX)' or '8.5* (XXX)'
		// We must define tva_tx, vat code
		$vatratecode = '';
		$tva_tx = preg_replace('/[^0-9\.].*$/', '', $tva_tx_txt); // keep remove all after the numbers and dot
		if (preg_match('/\((.*)\)/', $tva_tx_txt, $reg)) {
			$vatratecode = $reg[1];
		}

		$accountancy_code_sell = GETPOST('accountancy_code_sell', 'alpha');
		$accountancy_code_sell_intra = GETPOST('accountancy_code_sell_intra', 'alpha');
		$accountancy_code_sell_export = GETPOST('accountancy_code_sell_export', 'alpha');
		$accountancy_code_buy = GETPOST('accountancy_code_buy', 'alpha');
		$accountancy_code_buy_intra = GETPOST('accountancy_code_buy_intra', 'alpha');
		$accountancy_code_buy_export = GETPOST('accountancy_code_buy_export', 'alpha');
		if (empty($accountancy_code_sell) || $accountancy_code_sell == '-1') {
			$accountancy_code_sell = '';
		}
		if (empty($accountancy_code_sell_intra) || $accountancy_code_sell_intra == '-1') {
			$accountancy_code_sell_intra = '';
		}
		if (empty($accountancy_code_sell_export) || $accountancy_code_sell_export == '-1') {
			$accountancy_code_sell_export = '';
		}
		if (empty($accountancy_code_buy) || $accountancy_code_buy == '-1') {
			$accountancy_code_buy = '';
		}
		if (empty($accountancy_code_buy_intra) || $accountancy_code_buy_intra == '-1') {
			$accountancy_code_buy_intra = '';
		}
		if (empty($accountancy_code_buy_export) || $accountancy_code_buy_export == '-1') {
			$accountancy_code_buy_export = '';
		}

		$key_values = $productaccountancycode->getKeyValuesFromPost();
		if (!is_array($key_values)) {
			$error++;
		}

		if (empty($accountancy_code_sell) && empty($accountancy_code_sell_intra) && empty($accountancy_code_sell_export) &&
			empty($accountancy_code_buy) && empty($accountancy_code_buy_intra) && empty($accountancy_code_buy_export)
		) {
			setEventMessages($langs->trans('ErrorAtLeastOneAccountancyCodeRequired'), null, 'errors');
			$error++;
		}

		if (!$error) {
			$result = $productaccountancycode->addAccountancyCode($user, $product_id, $product_type, $free_line, $vatratecode, $tva_tx, $key_values,
				$accountancy_code_sell, $accountancy_code_sell_intra, $accountancy_code_sell_export, $accountancy_code_buy, $accountancy_code_buy_intra, $accountancy_code_buy_export);
			if ($result > 0) {
				if (!empty($backtopage)) {
					$backtopage = preg_replace('/--IDFORBACKTOPAGE--/', $product_id, $backtopage); // New method to autoselect project after a New on another form object creation
					if (preg_match('/\?/', $backtopage)) {
						$backtopage .= '&id=' . $product_id; // Old method
					}
					header("Location: " . $backtopage);
					exit;
				} else {
					header("Location: " . $_SERVER['PHP_SELF'] . ($is_product_card ? "?id=" . $product_id : ''));
					exit;
				}
			} else {
				if (count($productaccountancycode->errors)) {
					setEventMessages($productaccountancycode->error, $productaccountancycode->errors, 'errors');
				} else {
					setEventMessages($langs->trans($productaccountancycode->error), null, 'errors');
				}
			}
		}

		$action = "createline";
	} elseif ($action == 'updateline' && $lineid > 0 && $usercancreate) {
		// Update a product accountancy code
		$error = 0;

		$product_id = $is_product_card ? $object->id : GETPOST('product_id', 'int');
		$product_type = $product_id > 0 ? -1 : GETPOST('product_type', 'int');
		$free_line = $product_id > 0 ? 0 : GETPOST('free_line', 'int');

		if ($usercancreateonlyone) {
			// Test product type
			if ($product_id > 0) {
				$product_tmp = new Product($db);
				$result = $product_tmp->fetch($product_id);
				if ($result < 0) {
					setEventMessages($product_tmp->error, $product_tmp->errors, 'errors');
					$error++;
				} elseif (($product_tmp->type == Product::TYPE_PRODUCT && !$usercancreateproduct) || ($product_tmp->type == Product::TYPE_SERVICE && !$usercancreateservice)) {
					setEventMessage($langs->trans('ErrorEPAProductSelectedIsNotAuthorized'), 'errors');
					$error++;
				}
			} elseif (($product_type == Product::TYPE_PRODUCT && !$usercancreateproduct) || ($product_type == Product::TYPE_SERVICE && !$usercancreateservice)) {
				setEventMessage($langs->trans('ErrorEPAProductTypeSelectedIsNotAuthorized'), 'errors');
				$error++;
			}
		}

		$tva_tx_txt = GETPOST('tva_tx', 'alpha'); // tva_tx can be '8.5'  or  '8.5*'  or  '8.5 (XXX)' or '8.5* (XXX)'
		// We must define tva_tx, vat code
		$vatratecode = '';
		$tva_tx = preg_replace('/[^0-9\.].*$/', '', $tva_tx_txt); // keep remove all after the numbers and dot
		if (preg_match('/\((.*)\)/', $tva_tx_txt, $reg)) {
			$vatratecode = $reg[1];
		}

		$accountancy_code_sell = GETPOST('accountancy_code_sell', 'alpha');
		$accountancy_code_sell_intra = GETPOST('accountancy_code_sell_intra', 'alpha');
		$accountancy_code_sell_export = GETPOST('accountancy_code_sell_export', 'alpha');
		$accountancy_code_buy = GETPOST('accountancy_code_buy', 'alpha');
		$accountancy_code_buy_intra = GETPOST('accountancy_code_buy_intra', 'alpha');
		$accountancy_code_buy_export = GETPOST('accountancy_code_buy_export', 'alpha');
		if (empty($accountancy_code_sell) || $accountancy_code_sell == '-1') {
			$accountancy_code_sell = '';
		}
		if (empty($accountancy_code_sell_intra) || $accountancy_code_sell_intra == '-1') {
			$accountancy_code_sell_intra = '';
		}
		if (empty($accountancy_code_sell_export) || $accountancy_code_sell_export == '-1') {
			$accountancy_code_sell_export = '';
		}
		if (empty($accountancy_code_buy) || $accountancy_code_buy == '-1') {
			$accountancy_code_buy = '';
		}
		if (empty($accountancy_code_buy_intra) || $accountancy_code_buy_intra == '-1') {
			$accountancy_code_buy_intra = '';
		}
		if (empty($accountancy_code_buy_export) || $accountancy_code_buy_export == '-1') {
			$accountancy_code_buy_export = '';
		}

		$key_values = $productaccountancycode->getKeyValuesFromPost();
		if (!is_array($key_values)) {
			$error++;
		}

		if (empty($accountancy_code_sell) && empty($accountancy_code_sell_intra) && empty($accountancy_code_sell_export) &&
			empty($accountancy_code_buy) && empty($accountancy_code_buy_intra) && empty($accountancy_code_buy_export)
		) {
			setEventMessages($langs->trans('ErrorAtLeastOneAccountancyCodeRequired'), null, 'errors');
			$error++;
		}

		if (!$error) {
			$result = $productaccountancycode->updateAccountancyCode($lineid, $user, $product_id, $product_type, $free_line, $vatratecode, $tva_tx, $key_values,
				$accountancy_code_sell, $accountancy_code_sell_intra, $accountancy_code_sell_export, $accountancy_code_buy, $accountancy_code_buy_intra, $accountancy_code_buy_export);
			if ($result > 0) {
				if (!empty($backtopage)) {
					$backtopage = preg_replace('/--IDFORBACKTOPAGE--/', $product_id, $backtopage); // New method to autoselect project after a New on another form object creation
					if (preg_match('/\?/', $backtopage)) {
						$backtopage .= '&id=' . $product_id; // Old method
					}
					header("Location: " . $backtopage);
					exit;
				} else {
					header("Location: " . $_SERVER['PHP_SELF'] . ($is_product_card ? "?id=" . $product_id : ''));
					exit;
				}
			} else {
				if (count($productaccountancycode->errors)) {
					setEventMessages($productaccountancycode->error, $productaccountancycode->errors, 'errors');
				} else {
					setEventMessages($langs->trans($productaccountancycode->error), null, 'errors');
				}
			}
		}

		$action = "editline";
	} elseif ($action == 'confirm_deleteline' && $confirm == 'yes' && $usercancreate) {
		$error = 0;

		if ($usercancreateonlyone) {
			// Test product type
			$productaccountancycode = new ProductAccountancyCode($this->db);
			$result = $productaccountancycode->fetch($id);
			if ($result > 0) {
				if ($productaccountancycode->fk_product > 0) {
					$product_tmp = new Product($db);
					$result = $product_tmp->fetch($productaccountancycode->fk_product);
					if ($result < 0) {
						setEventMessages($product_tmp->error, $product_tmp->errors, 'errors');
						$error++;
					} elseif (($product_tmp->type == Product::TYPE_PRODUCT && !$usercancreateproduct) || ($product_tmp->type == Product::TYPE_SERVICE && !$usercancreateservice)) {
						setEventMessage($langs->trans('ErrorEPALineSelectedIsNotAuthorized'), 'errors');
						$error++;
					}
				} elseif (($productaccountancycode->product_type == Product::TYPE_PRODUCT && !$usercancreateproduct) || ($productaccountancycode->product_type == Product::TYPE_SERVICE && !$usercancreateservice)) {
					setEventMessage($langs->trans('ErrorEPALineSelectedIsNotAuthorized'), 'errors');
					$error++;
				}
			}
		}

		// Delete a product accountancy code
		if (!$error) {
			$result = $productaccountancycode->deleteAccountancyCode($user, $lineid);
			if ($result < 0) {
				$langs->load("errors");
				setEventMessages($productaccountancycode->error, $productaccountancycode->errors, 'errors');
			}
		}

		header('Location: ' . $_SERVER["PHP_SELF"] . ($is_product_card ? "?id=" . $object->id : ''));
		exit();
	}
}


/*
 *	View
 */

$form = new Form($db);
$formaccounting = new FormAccounting($db);

$help_url = '';
if (GETPOST("type") == '0' || ($object->type == Product::TYPE_PRODUCT)) {
	$help_url = 'EN:Module_Products|FR:Module_Produits|ES:M&oacute;dulo_Productos';
}
if (GETPOST("type") == '1' || ($object->type == Product::TYPE_SERVICE)) {
	$help_url = 'EN:Module_Services_En|FR:Module_Services|ES:M&oacute;dulo_Servicios';
}

$title = $langs->trans('ProductServiceCard');
$shortlabel = dol_trunc($object->label, 16);
if (GETPOST("type") == '0' || ($object->type == Product::TYPE_PRODUCT)) {
	$title = $langs->trans('Product')." ".$shortlabel." - ".$langs->trans('Notes');
	$help_url = 'EN:Module_Products|FR:Module_Produits|ES:M&oacute;dulo_Productos';
}
if (GETPOST("type") == '1' || ($object->type == Product::TYPE_SERVICE)) {
	$title = $langs->trans('Service')." ".$shortlabel." - ".$langs->trans('Notes');
	$help_url = 'EN:Module_Services_En|FR:Module_Services|ES:M&oacute;dulo_Servicios';
}

llxHeader('', $title, $help_url);

$formconfirm = '';

if ($action == 'ask_deleteline') {
	// Confirmation delete product/service line
	$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?lineid=' . $lineid . ($is_product_card ? "&id=" . $object->id : ''), $langs->trans('DeleteAccountancyCodes'), $langs->trans('ConfirmDeleteProductAccountLine'), 'confirm_deleteline', '', 0, 1);
}

// Call Hook formConfirm
$parameters = array('formConfirm' => $formconfirm, 'lineid' => $lineid);
$reshook = $hookmanager->executeHooks('formConfirm', $parameters, $object, $action); // Note that $action and $object may have been modified by hook
if (empty($reshook)) {
	$formconfirm .= $hookmanager->resPrint;
} elseif ($reshook > 0) {
	$formconfirm = $hookmanager->resPrint;
}

// Print form confirm
print $formconfirm;

if ($is_product_card) {
	/*
	 * Affichage onglets
	 */
	$head = product_prepare_head($object);
	$titre = $langs->trans("CardProduct" . $object->type);
	$picto = ($object->type == Product::TYPE_SERVICE ? 'service' : 'product');

	print dol_get_fiche_head($head, 'accountancy_code', $titre, -1, $picto);

	$linkback = '<a href="' . DOL_URL_ROOT . '/product/list.php?restore_lastsearch_values=1">' . $langs->trans("BackToList") . '</a>';
	$object->next_prev_filter = " fk_product_type = " . $object->type;

	$shownav = 1;
	if ($user->socid && !in_array('product', explode(',', $conf->global->MAIN_MODULES_FOR_EXTERNAL))) {
		$shownav = 0;
	}

	dol_banner_tab($object, 'ref', $linkback, $shownav, 'ref');

	print '<div class="fichecenter">';
	print '<div class="underbanner clearboth"></div>';
	print '</div>';
	print '<div style="clear:both"></div>';

	print dol_get_fiche_end();

	/*
	 * Action bar
	 */
	if (!$action || $action == 'createline' || $action == 'editline') {
		print "\n" . '<div class="tabsAction">' . "\n";

		$parameters = array();
		$reshook = $hookmanager->executeHooks('addMoreActionsButtons', $parameters, $object, $action); // Note that $action and $object may have been
		if (empty($reshook)) {
		}

		print "\n</div>\n";
	}
}

/*
 * Create / edit accountancy code
 */
if (($action == 'createline' || $action == 'editline') && $usercancreate) {
	print load_fiche_titre($langs->trans($action == 'createline' ? "CreateAccountancyCodes" : 'EditAccountancyCodes'), '');

	print '<form action="' . $_SERVER["PHP_SELF"] . ($is_product_card ? '?id=' . $object->id : '') . '" method="POST">';
	print '<input type="hidden" name="token" value="' . newToken() . '">';
	print '<input type="hidden" name="action" value="' . ($action == 'createline' ? 'addline' : 'updateline') . '">';
	if ($action == 'editline') {
		print '<input type="hidden" name="lineid" value="' . $lineid . '">';
	}

	print dol_get_fiche_head('');

	print '<table class="border centpercent">';

	// Product
	if (!$is_product_card) {
		print '<tr><td class="titlefieldcreate">' . $langs->trans("Product") . " | " . $langs->trans("Service") . '</td><td>';
		$default_product = empty($lineid) ? 0 : $productaccountancycode->fk_product;
		$product_id = (GETPOSTISSET('product_id') ? GETPOST('product_id', 'int') : $default_product);
		print $form->select_produits($product_id, 'product_id', $usercancreateonlyone ? ($usercancreateproduct ? Product::TYPE_PRODUCT : Product::TYPE_SERVICE) : '');
		print '</td></tr>';

		print '<tr><td class="titlefieldcreate">' . $langs->trans('or') . ' ' . $langs->trans("ProductType") . '</td><td>';
        $default_product_type = empty($lineid) ? 0 : $productaccountancycode->product_type;
        $product_type = (GETPOSTISSET('product_type') ? GETPOST('product_type', 'int') : $default_product_type);
        $default_free_line = empty($lineid) ? 0 : $productaccountancycode->free_line;
        $free_line = (GETPOSTISSET('free_line') ? ((GETPOST('free_line', 'int') > 0) ? ' checked="checked"' : '') : (($default_free_line > 0) ? ' checked="checked"' : ''));
        print $form->selectarray('product_type', $global_product_types, $product_type);
		print '&nbsp;<input type="checkbox" id="free_line" name="free_line" value="1"' . $free_line . '><label for="free_line">' . $langs->trans("FreeLine") . '</label>';
		print '</td></tr>';
	}

	// VAT
	print '<tr><td class="titlefieldcreate">' . $langs->trans("VATRate") . '</td><td>';
	$default_vat = empty($lineid) ? get_default_tva($mysoc, $mysoc) : ($productaccountancycode->vat_code ? $productaccountancycode->tva_tx . ' (' . $productaccountancycode->vat_code . ')' : $productaccountancycode->tva_tx);
	$vat = (GETPOSTISSET('tva_tx') ? GETPOST('tva_tx', 'alpha') : $default_vat);
	print $form->load_tva("tva_tx", $vat, $mysoc, '', $object->id, 0, $object->type, false, 1);
	print '</td></tr>';

	// Key Values
	$productaccountancycode->printKeyValuesListCreateEditInput();

	// Accountancy code sell
	print '<tr><td class="titlefieldcreate">'.$langs->trans("ProductAccountancySellCode").'</td><td>';
	$default_accountancy_code = !empty($conf->accounting->enabled) && empty($lineid) ? ($object->type == Product::TYPE_PRODUCT ? $conf->global->ACCOUNTING_PRODUCT_SOLD_ACCOUNT : $conf->global->ACCOUNTING_SERVICE_SOLD_ACCOUNT) : $productaccountancycode->accountancy_code_sell;
	$accountancy_code = (GETPOSTISSET('accountancy_code_sell') ? GETPOST('accountancy_code_sell', 'alpha') : $default_accountancy_code);
	if (!empty($conf->accounting->enabled)) {
		print $formaccounting->select_account($accountancy_code, 'accountancy_code_sell', 1, null, 1, 1, 'minwidth150 maxwidth300', 1);
	} else {
		print '<input class="minwidth150" name="accountancy_code_sell" value="' . $accountancy_code . '">';
	}
	print '</td></tr>';

	// Accountancy code sell intra
	if ($isInEEC) {
		print '<tr><td class="titlefieldcreate">'.$langs->trans("ProductAccountancySellIntraCode").'</td><td>';
		$default_accountancy_code = !empty($conf->accounting->enabled) && empty($lineid) ? ($object->type == Product::TYPE_PRODUCT ? $conf->global->ACCOUNTING_PRODUCT_SOLD_INTRA_ACCOUNT : $conf->global->ACCOUNTING_SERVICE_SOLD_INTRA_ACCOUNT) : $productaccountancycode->accountancy_code_sell_intra;
		$accountancy_code = (GETPOSTISSET('accountancy_code_sell_intra') ? GETPOST('accountancy_code_sell_intra', 'alpha') : $default_accountancy_code);
		if (!empty($conf->accounting->enabled)) {
			print $formaccounting->select_account($accountancy_code, 'accountancy_code_sell_intra', 1, null, 1, 1, 'minwidth150 maxwidth300', 1);
		} else {
			print '<input class="minwidth150" name="accountancy_code_sell_intra" value="' . $accountancy_code . '">';
		}
		print '</td></tr>';
	}

	// Accountancy code sell export
	print '<tr><td class="titlefieldcreate">'.$langs->trans("ProductAccountancySellExportCode").'</td><td>';
	$default_accountancy_code = !empty($conf->accounting->enabled) && empty($lineid) ? ($object->type == Product::TYPE_PRODUCT ? $conf->global->ACCOUNTING_PRODUCT_SOLD_EXPORT_ACCOUNT : $conf->global->ACCOUNTING_SERVICE_SOLD_EXPORT_ACCOUNT) : $productaccountancycode->accountancy_code_sell_export;
	$accountancy_code = (GETPOSTISSET('accountancy_code_sell_export') ? GETPOST('accountancy_code_sell_export', 'alpha') : $default_accountancy_code);
	if (!empty($conf->accounting->enabled)) {
		print $formaccounting->select_account($accountancy_code, 'accountancy_code_sell_export', 1, null, 1, 1, 'minwidth150 maxwidth300', 1);
	} else {
		print '<input class="minwidth150" name="accountancy_code_sell_export" value="' . $accountancy_code . '">';
	}
	print '</td></tr>';

	// Accountancy code buy
	print '<tr><td>'.$langs->trans("ProductAccountancyBuyCode").'</td><td>';
	$default_accountancy_code = !empty($conf->accounting->enabled) && empty($lineid) ? ($object->type == Product::TYPE_PRODUCT ? $conf->global->ACCOUNTING_PRODUCT_BUY_ACCOUNT : $conf->global->ACCOUNTING_SERVICE_BUY_ACCOUNT) : $productaccountancycode->accountancy_code_buy;
	$accountancy_code = (GETPOSTISSET('accountancy_code_buy') ? GETPOST('accountancy_code_buy', 'alpha') : $default_accountancy_code);
	if (!empty($conf->accounting->enabled)) {
		print $formaccounting->select_account($accountancy_code, 'accountancy_code_buy', 1, null, 1, 1, 'minwidth150 maxwidth300', 1);
	} else {
		print '<input class="minwidth150" name="accountancy_code_buy" value="' . $accountancy_code . '">';
	}
	print '</td></tr>';

	// Accountancy code buy intra
	if ($isInEEC) {
		print '<tr><td class="titlefieldcreate">'.$langs->trans("ProductAccountancyBuyIntraCode").'</td><td>';
		$default_accountancy_code = !empty($conf->accounting->enabled) && empty($lineid) ? ($object->type == Product::TYPE_PRODUCT ? $conf->global->ACCOUNTING_PRODUCT_BUY_INTRA_ACCOUNT : $conf->global->ACCOUNTING_SERVICE_BUY_INTRA_ACCOUNT) : $productaccountancycode->accountancy_code_buy_intra;
		$accountancy_code = (GETPOSTISSET('accountancy_code_buy_intra') ? GETPOST('accountancy_code_buy_intra', 'alpha') : $default_accountancy_code);
		if (!empty($conf->accounting->enabled)) {
			print $formaccounting->select_account($accountancy_code, 'accountancy_code_buy_intra', 1, null, 1, 1, 'minwidth150 maxwidth300', 1);
		} else {
			print '<input class="minwidth150" name="accountancy_code_buy_intra" value="' . $accountancy_code . '">';
		}
		print '</td></tr>';
	}

	// Accountancy code buy export
	print '<tr><td class="titlefieldcreate">'.$langs->trans("ProductAccountancyBuyExportCode").'</td><td>';
	$default_accountancy_code = !empty($conf->accounting->enabled) && empty($lineid) ? ($object->type == Product::TYPE_PRODUCT ? $conf->global->ACCOUNTING_PRODUCT_BUY_EXPORT_ACCOUNT : $conf->global->ACCOUNTING_SERVICE_BUY_EXPORT_ACCOUNT) : $productaccountancycode->accountancy_code_buy_export;
	$accountancy_code = (GETPOSTISSET('accountancy_code_buy_export') ? GETPOST('accountancy_code_buy_export', 'alpha') : $default_accountancy_code);
	if (!empty($conf->accounting->enabled)) {
		print $formaccounting->select_account($accountancy_code, 'accountancy_code_buy_export', 1, null, 1, 1, 'minwidth150 maxwidth300', 1);
	} else {
		print '<input class="minwidth150" name="accountancy_code_buy_export" value="' . $accountancy_code . '">';
	}
	print '</td></tr>';

	print '</table>';

	print dol_get_fiche_end();

	print '<div class="center">';
	print '<input type="submit" class="button button-save" value="' . $langs->trans("Save") . '">';
	print '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
	print '<input type="submit" class="button button-cancel" name="cancel" value="' . $langs->trans("Cancel") . '">';
	print '</div>';

	print '<br></form><br>';
} else {
	/*
	 * View accountancy codes list
	 */

	$sql = "SELECT pac.rowid, pac.vat_code, pac.tva_tx";
	$sql .= ", pac.accountancy_code_sell, pac.accountancy_code_sell_intra, pac.accountancy_code_sell_export";
	$sql .= ", pac.accountancy_code_buy, pac.accountancy_code_buy_intra, pac.accountancy_code_buy_export";
	if (!$is_product_card) {
		$sql .= ", pac.fk_product, pac.product_type, pac.free_line";
	}
	if (!empty($key_conf) && !empty($key_conf['list_sql_select'])) $sql .= ',' . $key_conf['list_sql_select'];
	// Add fields from hooks
	$parameters = array();
	$reshook = $hookmanager->executeHooks('printFieldListSelect', $parameters, $object); // Note that $action and $object may have been modified by hook
	$sql .= $hookmanager->resPrint;
	$sql .= " FROM " . MAIN_DB_PREFIX . "product_accountancy_code as pac";
	if (!$is_product_card) $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "product as p ON p.rowid = pac.fk_product";
	if (!empty($key_conf) && !empty($key_conf['list_sql_from'])) $sql .= ' ' . $key_conf['list_sql_from'];
	// Add table from hooks
	$parameters = array();
	$reshook = $hookmanager->executeHooks('printFieldListFrom', $parameters, $object); // Note that $action and $object may have been modified by hook
	$sql .= $hookmanager->resPrint;
	$sql .= " WHERE pac.entity = " . (empty($conf->global->MAIN_PRODUCT_PERENTITY_SHARED) && $is_product_card ? $object->entity : $conf->entity);
	if ($usercanreadonlyone) {
		if ($usercanreadproduct) {
			$sql .= " AND (pac.product_type = " . Product::TYPE_PRODUCT . " OR p.fk_product_type = " . Product::TYPE_PRODUCT . ")";
		} else {
			$sql .= " AND (pac.product_type = " . Product::TYPE_SERVICE . " OR p.fk_product_type = " . Product::TYPE_SERVICE . ")";
		}
	}
	if ($is_product_card) $sql .= " AND pac.fk_product = " . $object->id;
	if (!$is_product_card && $search_product) {
		$sql .= natural_search('p.ref', $search_product);
	}
	if (!$is_product_card && $search_product_type >= 0) {
		$sql .= " AND pac.product_type = " . $search_product_type;
	}
	if (!$is_product_card && $search_free_line >= 0) {
		$sql .= " AND pac.free_line " . ($search_free_line ? '= 1' : 'IS NULL');
	}
	if ($search_vatrate) {
		$sql .= natural_search('pac.tva_tx', $search_vatrate, 1);
	}
	$sql .= $productaccountancycode->getListSearchSQL($search_key_values);
	// Add where from hooks
	$parameters = array();
	$reshook = $hookmanager->executeHooks('printFieldListWhere', $parameters, $object); // Note that $action and $object may have been modified by hook
	$sql .= $hookmanager->resPrint;
	$sql .= $db->order($sortfield, $sortorder);

	$nbtotalofrecords = '';
	if (empty($conf->global->MAIN_DISABLE_FULL_SCANLIST)) {
		$result = $db->query($sql);
		$nbtotalofrecords = $db->num_rows($result);
		if (($page * $limit) > $nbtotalofrecords) {    // if total resultset is smaller then paging size (filtering), goto and load page 0
			$page = 0;
			$offset = 0;
		}
	}

	$sql .= $db->plimit($limit + 1, $offset);

	$resql = $db->query($sql);
	if ($resql) {
		$num = $db->num_rows($resql);

		$param = '';
		if ($is_product_card) {
			$param .= '&id=' . $object->id;
		}
		if ($limit > 0 && $limit != $conf->liste_limit) {
			$param .= '&limit=' . urlencode($limit);
		}
		if (!$is_product_card && $search_product) {
			$param .= "&search_product=" . urlencode($search_product);
		}
		if ($search_vatrate) {
			$param .= "&search_vatrate=" . urlencode($search_vatrate);
		}
		$param .= $productaccountancycode->getKeyValuesListSearchParam($search_key_values);
		if ($optioncss != '') {
			$param .= '&optioncss=' . urlencode($optioncss);
		}

		$massactionbutton = '';
		$newcardbutton = dolGetButtonTitle($langs->trans('CreateAccountancyCodes'), '', 'fa fa-plus-circle', $_SERVER['PHP_SELF'] . '?action=createline' . ($is_product_card ? '&id=' . $object->id : ''), '', $usercancreate);

		print '<form action="' . $_SERVER["PHP_SELF"] . ($is_product_card ? '?id=' . $object->id : '') . '" method="POST" name="formulaire">';
		if ($optioncss != '') {
			print '<input type="hidden" name="optioncss" value="' . $optioncss . '">';
		}
		print '<input type="hidden" name="formfilteraction" id="formfilteraction" value="list">';
		print '<input type="hidden" name="action" value="list">';
		print '<input type="hidden" name="token" value="' . newToken() . '">';
		print '<input type="hidden" name="sortfield" value="' . $sortfield . '">';
		print '<input type="hidden" name="sortorder" value="' . $sortorder . '">';

		print_barre_liste($langs->trans('AccountancyCodes'), $page, $_SERVER['PHP_SELF'], $param, $sortfield, $sortorder, $massactionbutton, $num, $nbtotalofrecords, 'title_accountancy.png', 0, $newcardbutton, '', $limit, 0, 0, 1);

		$colspan = 6;
		if ($isInEEC) $colspan += 2;
		if (!$is_product_card) $colspan += 1;
		if (!empty($key_conf)) $colspan += $key_conf['list_colspan'];

		print '<div class="div-table-responsive-no-min">';
		print '<table class="liste centpercent">';

		// Filters lines
		// --------------------------------------------------------------------
		print '<tr class="liste_titre_filter">';
		if (!$is_product_card) {
			print '<td class="liste_titre left"><input class="flat" size="4" type="text" name="search_product" value="' . $search_product . '"></td>';

			print '<td class="liste_titre left">';
			print $form->selectarray('search_product_type', $global_product_types, $search_product_type, 1);
			print '</td>';
			print '<td class="liste_titre left">';
			print $form->selectyesno('search_free_line', $search_free_line, 1, false, 1);
			print '</td>';
		}
		print '<td class="liste_titre"><input class="flat" size="4" type="text" name="search_vatrate" value="' . $search_vatrate . '"></td>';
		$productaccountancycode->printKeyValuesListSearch($search_key_values);
		print '<td class="liste_titre">&nbsp;</td>';
		if ($isInEEC) print '<td class="liste_titre">&nbsp;</td>';
		print '<td class="liste_titre">&nbsp;</td>';
		print '<td class="liste_titre">&nbsp;</td>';
		if ($isInEEC) print '<td class="liste_titre">&nbsp;</td>';
		print '<td class="liste_titre">&nbsp;</td>';
		// Fields from hook
		$parameters = array();
		$reshook = $hookmanager->executeHooks('printFieldListOption', $parameters); // Note that $action and $object may have been modified by hook
		print $hookmanager->resPrint;
		print '<td class="liste_titre maxwidthsearch">';
		$searchpicto = $form->showFilterAndCheckAddButtons(0);
		print $searchpicto;
		print '</td>';
		print "</tr>\n";

		// Fields title label
		// --------------------------------------------------------------------
		print '<tr class="liste_titre">';
		if (!$is_product_card) {
			print_liste_field_titre($langs->trans("Product") . " | " . $langs->trans("Service"), $_SERVER["PHP_SELF"], "p.ref", "", $param, "", $sortfield, $sortorder);
			print_liste_field_titre($langs->trans("ProductType"), $_SERVER["PHP_SELF"], "pac.product_type", "", $param, "", $sortfield, $sortorder);
			print_liste_field_titre($langs->trans("FreeLine"), $_SERVER["PHP_SELF"], "pac.free_line", "", $param, "", $sortfield, $sortorder);
		}
		print_liste_field_titre("VATRate", $_SERVER["PHP_SELF"], "pac.tva_tx", "", $param, "", $sortfield, $sortorder);
		$productaccountancycode->printKeyValuesListTitle($param, $sortfield, $sortorder);
		print_liste_field_titre("ProductAccountancySellCode", $_SERVER["PHP_SELF"], "pac.accountancy_code_sell", "", $param, '', $sortfield, $sortorder);
		if ($isInEEC) print_liste_field_titre("ProductAccountancySellIntraCode", $_SERVER["PHP_SELF"], "pac.accountancy_code_sell_intra", "", $param, '', $sortfield, $sortorder);
		print_liste_field_titre("ProductAccountancySellExportCode", $_SERVER["PHP_SELF"], "pac.accountancy_code_sell_export", "", $param, '', $sortfield, $sortorder);
		print_liste_field_titre("ProductAccountancyBuyCode", $_SERVER["PHP_SELF"], "pac.accountancy_code_buy", "", $param, '', $sortfield, $sortorder);
		if ($isInEEC) print_liste_field_titre("ProductAccountancyBuyIntraCode", $_SERVER["PHP_SELF"], "pac.accountancy_code_buy_intra", "", $param, '', $sortfield, $sortorder);
		print_liste_field_titre("ProductAccountancyBuyExportCode", $_SERVER["PHP_SELF"], "pac.accountancy_code_export", "", $param, '', $sortfield, $sortorder);
		// Hook fields
		$parameters = array('param' => $param, 'sortfield' => $sortfield, 'sortorder' => $sortorder);
		$reshook = $hookmanager->executeHooks('printFieldListTitle', $parameters); // Note that $action and $object may have been modified by hook
		print $hookmanager->resPrint;
		print_liste_field_titre('', $_SERVER["PHP_SELF"], "", '', '', '', $sortfield, $sortorder, 'maxwidthsearch ');
		print "</tr>\n";

		// Fields values
		// --------------------------------------------------------------------
		if ($num > 0) {
			$i = 0;
			while ($i < min($num, $limit)) {
				$obj = $db->fetch_object($resql);

				print '<tr class="oddeven" id="line_' . $obj->rowid . '">';

				// Product
				if (!$is_product_card) {
					print '<td class="nowraponall">';
					if ($obj->fk_product > 0) {
						if (!isset($product_cached[$obj->fk_product])) {
							$product_tmp = new Product($db);
							$product_tmp->fetch($obj->fk_product);
							$product_cached[$obj->fk_product] = $product_tmp;
						}
						print $product_cached[$obj->fk_product]->getNomUrl(1);
					}
					print "</td>\n";

					print '<td class="nowraponall">';
					if ($obj->product_type >= 0) {
						print $global_product_types[$obj->product_type];
					}
					print "</td>\n";

					print '<td class="nowraponall">';
					if (!empty($obj->free_line)) {
						print yn($obj->free_line);
					}
					print "</td>\n";
				}

				// VAT
				print '<td class="nowraponall">';
				print vatrate($obj->tva_tx, true);
				print "</td>\n";

				// Key
				$productaccountancycode->printKeyValuesListValue($obj);

				// Accountancy code sell
				print '<td class="nowrap">' . $productaccountancycode->getAccountancyCodeLabel($obj->accountancy_code_sell) . '</td>';
				if ($isInEEC) print '<td class="nowrap">' . $productaccountancycode->getAccountancyCodeLabel($obj->accountancy_code_sell_intra) . '</td>';
				print '<td class="nowrap">' . $productaccountancycode->getAccountancyCodeLabel($obj->accountancy_code_sell_export) . '</td>';

				// Accountancy code buy
				print '<td class="nowrap">' . $productaccountancycode->getAccountancyCodeLabel($obj->accountancy_code_buy) . '</td>';
				if ($isInEEC) print '<td class="nowrap">' . $productaccountancycode->getAccountancyCodeLabel($obj->accountancy_code_buy_intra) . '</td>';
				print '<td class="nowrap">' . $productaccountancycode->getAccountancyCodeLabel($obj->accountancy_code_buy_export) . '</td>';

				// Fields from hook
				$parameters = array('obj' => $obj, 'i' => $i);
				$reshook = $hookmanager->executeHooks('printFieldListValue', $parameters); // Note that $action and $object may have been modified by hook
				print $hookmanager->resPrint;

				// Action
				print '<td class="nowrap center">';
				if ($usercancreate && (
						empty($usercancreateonlyone) ||
						($usercancreateproduct && ($obj->fk_product > 0 ? $product_cached[$obj->fk_product]->type == Product::TYPE_PRODUCT : $obj->product_type == Product::TYPE_PRODUCT)) ||
						($usercancreateservice && ($obj->fk_product > 0 ? $product_cached[$obj->fk_product]->type == Product::TYPE_SERVICE : $obj->product_type == Product::TYPE_SERVICE))
					)
				) {
					print '<a class="editfielda reposition" href="' . $_SERVER["PHP_SELF"] . '?action=editline&amp;lineid=' . $obj->rowid . ($is_product_card ? '&amp;id=' . $object->id : '') . '#line_' . $obj->rowid . '">' . img_edit() . '</a>';
					print '&nbsp;&nbsp;';
					print '<a class="reposition" href="' . $_SERVER["PHP_SELF"] . '?action=ask_deleteline&amp;lineid=' . $obj->rowid . ($is_product_card ? '&amp;id=' . $object->id : '') . '">' . img_delete() . '</a>';
				}
				print '</td>';

				print "</tr>\n";
				$i++;
			}
		} elseif ($num == 0) {
			// If no record found
			print '<tr><td colspan="' . $colspan . '" class="opacitymedium">' . $langs->trans("NoRecordFound") . '</td></tr>';
		}

		$db->free($resql);

		$parameters = array('sql' => $sql);
		$reshook = $hookmanager->executeHooks('printFieldListFooter', $parameters); // Note that $action and $object may have been modified by hook
		print $hookmanager->resPrint;

		print "</table>\n";
		print "</div>";
		print '</form>';
	} else {
		dol_print_error($db);
	}
}

// End of page
llxFooter();
$db->close();
