<?php
/* Copyright (C) 2023 Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	    \file       htdocs/extendedproductaccountancy/admin/setup.php
 *		\ingroup    extendedproductaccountancy
 *		\brief      Page to setup extendedproductaccountancy module
 */

// Change this following line to use the correct relative path (../, ../../, etc)
$res=0;
if (! $res && file_exists("../../main.inc.php")) $res=@include '../../main.inc.php';			// to work if your module directory is into a subdir of root htdocs directory
if (! $res && file_exists("../../../main.inc.php")) $res=@include '../../../main.inc.php';		// to work if your module directory is into a subdir of root htdocs directory
if (! $res) die("Include of main fails");
require_once DOL_DOCUMENT_ROOT.'/core/lib/admin.lib.php';
dol_include_once('/extendedproductaccountancy/lib/extendedproductaccountancy.lib.php');
dol_include_once('/extendedproductaccountancy/class/productaccountancycode.class.php');

$langs->load("admin");
$langs->load("extendedproductaccountancy@extendedproductaccountancy");
$langs->load("opendsi@extendedproductaccountancy");

if (!$user->admin) accessforbidden();

$action = GETPOST('action', 'aZ09');
$productaccountancycode = new ProductAccountancyCode($db);

/*
 *	Actions
 */

$errors = [];
$error = 0;
$db->begin();

if ($action == 'set_options') {
	$constvalue = GETPOST('ACCOUNTING_PRODUCT_ACCOUNTANCY_CODE_ADDITIONAL_KEY', 'alphanohtml');
	if ($constvalue < 0) $constvalue = 0;
	if (!dolibarr_set_const($db, 'ACCOUNTING_PRODUCT_ACCOUNTANCY_CODE_ADDITIONAL_KEY', $constvalue, 'chaine', 0, '', $conf->entity)) {
		$error++;
	} else {
		$productaccountancycode->initialize(true);
		$result = $productaccountancycode->createKeyValueColumns();
		if ($result < 0) {
			setEventMessages($productaccountancycode->error, $productaccountancycode->errors, 'errors');
			$error++;
		}
	}
	if ($error) {
		setEventMessages($langs->trans("Error"), null, 'errors');
	}
} elseif (preg_match('/set_(.*)/',$action,$reg)) {
    $code = $reg[1];
    $value = (GETPOST($code) ? GETPOST($code) : 1);
	$res = dolibarr_set_const($db, $code, $value, 'chaine', 0, '', $conf->entity);
	if ($res < 0) {
        $errors[] = $db->lasterror();
        $error++;
    }
} elseif (preg_match('/del_(.*)/',$action,$reg)) {
    $code = $reg[1];
	$res = dolibarr_del_const($db, $code, $conf->entity);
    if ($res < 0) {
        $errors[] = $db->lasterror();
        $error++;
    }
}

if ($action != '') {
    if (!$error) {
	    $db->commit();
        setEventMessage($langs->trans("SetupSaved"));
	    header("Location: " . $_SERVER["PHP_SELF"]);
        exit;
    } else {
	    $db->rollback();
        setEventMessages(/*$langs->trans("Error")*/'', $errors, 'errors');
    }
}


/*
 *	View
 */

$form = new Form($db);

$wikihelp='';
llxHeader('', $langs->trans("ExtendedProductAccountancySetup"), $wikihelp);

$linkback='<a href="'.DOL_URL_ROOT.'/admin/modules.php">'.$langs->trans("BackToModuleList").'</a>';
print load_fiche_titre($langs->trans("ExtendedProductAccountancySetup"),$linkback,'title_setup');
print "<br>\n";

$head=extendedproductaccountancy_admin_prepare_head();

dol_fiche_head($head, 'settings', $langs->trans("Module163073Name"), 0, 'opendsi@extendedproductaccountancy');

/**
 * Settings.
 */

print '<div id="options"></div>';

print '<form method="post" action="'.$_SERVER["PHP_SELF"].'">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="set_options">';

print '<div class="div-table-responsive-no-min">';
print '<table class="noborder centpercent">';
print '<tr class="liste_titre">';
print '<td class="width20p">'.$langs->trans("Parameters").'</td>'."\n";
print '<td>'.$langs->trans("Description").'</td>'."\n";
print '<td class="right">'.$langs->trans("Value").'</td>'."\n";
print "</tr>\n";

print '<tr class="oddeven">';
print '<td>' . $langs->trans("ACCOUNTING_PRODUCT_ACCOUNTANCY_CODE_ADDITIONAL_KEY") . '</td>';
print '<td>' . $langs->trans("ACCOUNTING_PRODUCT_ACCOUNTANCY_CODE_ADDITIONAL_KEYDesc") . '</td>';
print '<td class="right">';
$array = $productaccountancycode->getKeyTypes();
print $form->selectarray("ACCOUNTING_PRODUCT_ACCOUNTANCY_CODE_ADDITIONAL_KEY", $array, $conf->global->ACCOUNTING_PRODUCT_ACCOUNTANCY_CODE_ADDITIONAL_KEY, $langs->trans('None'), 0, 0, '', 0, 0, 0, '', ' minwidth400');
print '</td>';
print '</tr>';

print '</table>'."\n";
print '</div>';

print '<br>';
print '<div class="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Save").'">';
print '</div>';

print '</form>';

print dol_get_fiche_end();

llxFooter();

$db->close();
