<?php
/* Copyright (C) 2023      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


class ActionsExtendedProductAccountancy
{
    /**
     * @var DoliDB Database handler.
     */
    public $db;
    /**
     * @var string Error
     */
    public $error = '';
    /**
     * @var array Errors
     */
    public $errors = array();

    /**
     * @var array Hook results. Propagated to $hookmanager->resArray for later reuse
     */
    public $results = array();

    /**
     * @var string String displayed by executeHook() immediately after return
     */
    public $resprints;

	/**
	 * @var boolean		If action validate history is already processed
	 */
	public static $not_validate_history = false;

    /**
     * Constructor
     *
     * @param        DoliDB $db Database handler
     */
    public function __construct($db)
    {
        $this->db = $db;
    }

	/**
	 * Overloading the updateSession function : replacing the parent's function with the one below
	 *
	 * @param	array()			$parameters		Hook metadatas (context, etc...)
	 * @param	CommonObject	&$object		The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			&$action		Current action (if set). Generally create or edit or null
	 * @param	HookManager		$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int								< 0 on error, 0 on success, 1 to replace standard code
	 */
	function updateSession($parameters, &$object, &$action, $hookmanager)
	{
		$this->validateHistory();
		return 0;
	}

	/**
	 * Overloading the afterLogin function : replacing the parent's function with the one below
	 *
	 * @param	array()			$parameters		Hook metadatas (context, etc...)
	 * @param	CommonObject	&$object		The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			&$action		Current action (if set). Generally create or edit or null
	 * @param	HookManager		$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int								< 0 on error, 0 on success, 1 to replace standard code
	 */
	function afterLogin($parameters, &$object, &$action, $hookmanager)
	{
		$this->validateHistory();
		return 0;
	}

	/**
	 * Overloading the printFieldListWhere function : replacing the parent's function with the one below
	 *
	 * @param	array()			$parameters		Hook metadatas (context, etc...)
	 * @param	CommonObject	&$object		The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			&$action		Current action (if set). Generally create or edit or null
	 * @param	HookManager		$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int								< 0 on error, 0 on success, 1 to replace standard code
	 */
	function printFieldListWhere($parameters, &$object, &$action, $hookmanager)
	{
		global $conf, $langs, $user, $mysoc;

		$context = explode(':', $parameters['context']);

		if (in_array('accountancycustomerlist', $context)) {
			global $sql, $chartaccountcode, $search_societe, $search_lineid, $search_invoice, $search_ref, $search_label, $search_desc, $search_amount, $search_account, $search_vat, $search_date_start, $search_date_end, $search_country, $search_tvaintra;

			$accountancy_code_sell_field = $this->db->ifsql("pac.accountancy_code_sell IS NULL OR pac.accountancy_code_sell = ''", 'p.accountancy_code_sell', 'pac.accountancy_code_sell');
			$accountancy_code_sell_intra_field = $this->db->ifsql("pac.accountancy_code_sell_intra IS NULL OR pac.accountancy_code_sell_intra = ''", 'p.accountancy_code_sell_intra', 'pac.accountancy_code_sell_intra');
			$accountancy_code_sell_export_field = $this->db->ifsql("pac.accountancy_code_sell_export IS NULL OR pac.accountancy_code_sell_export = ''", 'p.accountancy_code_sell_export', 'pac.accountancy_code_sell_export');
			$accountancy_code_buy_field = $this->db->ifsql("pac.accountancy_code_buy IS NULL OR pac.accountancy_code_buy = ''", 'p.accountancy_code_buy', 'pac.accountancy_code_buy');
			$accountancy_code_buy_intra_field = $this->db->ifsql("pac.accountancy_code_buy_intra IS NULL OR pac.accountancy_code_buy_intra = ''", 'p.accountancy_code_buy_intra', 'pac.accountancy_code_buy_intra');
			$accountancy_code_buy_export_field = $this->db->ifsql("pac.accountancy_code_buy_export IS NULL OR pac.accountancy_code_buy_export = ''", 'p.accountancy_code_buy_export', 'pac.accountancy_code_buy_export');

			// Customer Invoice lines
			$sql = "SELECT f.rowid as facid, f.ref, f.datef, f.type as ftype,";
			$sql .= " l.rowid, l.fk_product, l.description, l.total_ht, l.fk_code_ventilation, l.product_type as type_l, l.tva_tx as tva_tx_line, l.vat_src_code,";
			$sql .= " p.rowid as product_id, p.ref as product_ref, p.label as product_label, p.fk_product_type as type, p.tva_tx as tva_tx_prod,";
			$sql .= " " . $accountancy_code_sell_field . " as code_sell, " . $accountancy_code_sell_intra_field . " as code_sell_intra, " . $accountancy_code_sell_export_field . " as code_sell_export,";
			$sql .= " " . $accountancy_code_buy_field . " as code_buy, " . $accountancy_code_buy_intra_field . " as code_buy_intra, " . $accountancy_code_buy_export_field . " as code_buy_export,";
			$sql .= " p.tosell as status, p.tobuy as status_buy,";
			$sql .= " aa.rowid as aarowid, aa2.rowid as aarowid_intra, aa3.rowid as aarowid_export, aa4.rowid as aarowid_thirdparty,";
			$sql .= " co.code as country_code, co.label as country_label,";
			$sql .= " s.rowid as socid, s.nom as name, s.tva_intra, s.email, s.town, s.zip, s.fk_pays, s.client, s.fournisseur, s.code_client, s.code_fournisseur, s.code_compta as code_compta_client, s.code_compta_fournisseur,";
			if (!empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED)) {
				$sql .= " spe.accountancy_code_sell as company_code_sell";
			} else {
				$sql .= " s.accountancy_code_sell as company_code_sell";
			}
			$sql .= " FROM ".MAIN_DB_PREFIX."facture as f";
			$sql .= " INNER JOIN ".MAIN_DB_PREFIX."societe as s ON s.rowid = f.fk_soc";
			if (!empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED)) {
				$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "societe_perentity as spe ON spe.fk_soc = s.rowid AND spe.entity = " . ((int) $conf->entity);
			}
			$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."c_country as co ON co.rowid = s.fk_pays ";
			$sql .= " INNER JOIN ".MAIN_DB_PREFIX."facturedet as l ON f.rowid = l.fk_facture";
			$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."product as p ON p.rowid = l.fk_product";
			$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "product_accountancy_code as pac ON pac.rowid = EPA_GET_CUSTOMER_JOIN_ID(l.rowid)";
			$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."accounting_account as aa  ON " . $accountancy_code_sell_field . " = aa.account_number         AND aa.active = 1  AND aa.fk_pcg_version = '".$this->db->escape($chartaccountcode)."' AND aa.entity = ".$conf->entity;
			$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."accounting_account as aa2 ON " . $accountancy_code_sell_intra_field . " = aa2.account_number  AND aa2.active = 1 AND aa2.fk_pcg_version = '".$this->db->escape($chartaccountcode)."' AND aa2.entity = ".$conf->entity;
			$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."accounting_account as aa3 ON " . $accountancy_code_sell_export_field . " = aa3.account_number AND aa3.active = 1 AND aa3.fk_pcg_version = '".$this->db->escape($chartaccountcode)."' AND aa3.entity = ".$conf->entity;
			$alias_societe_perentity = empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED) ? "s" : "spe";
			$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."accounting_account as aa4 ON " . $alias_societe_perentity . ".accountancy_code_sell = aa4.account_number        AND aa4.active = 1 AND aa4.fk_pcg_version = '".$this->db->escape($chartaccountcode)."' AND aa4.entity = ".$conf->entity;

			$sql .= " WHERE f.fk_statut > 0 AND l.fk_code_ventilation <= 0";
			$sql .= " AND l.product_type <= 2";
			// Define begin binding date
			if (!empty($conf->global->ACCOUNTING_DATE_START_BINDING)) {
				$sql .= " AND f.datef >= '".$this->db->idate($conf->global->ACCOUNTING_DATE_START_BINDING)."'";
			}
			// Add search filter like
			if ($search_societe) {
				$sql .= natural_search('s.nom', $search_societe);
			}
			if ($search_lineid) {
				$sql .= natural_search("l.rowid", $search_lineid, 1);
			}
			if (strlen(trim($search_invoice))) {
				$sql .= natural_search("f.ref", $search_invoice);
			}
			if (strlen(trim($search_ref))) {
				$sql .= natural_search("p.ref", $search_ref);
			}
			if (strlen(trim($search_label))) {
				$sql .= natural_search("p.label", $search_label);
			}
			if (strlen(trim($search_desc))) {
				$sql .= natural_search("l.description", $search_desc);
			}
			if (strlen(trim($search_amount))) {
				$sql .= natural_search("l.total_ht", $search_amount, 1);
			}
			if (strlen(trim($search_account))) {
				$sql .= natural_search("aa.account_number", $search_account);
			}
			if (strlen(trim($search_vat))) {
				$sql .= natural_search("l.tva_tx", price2num($search_vat), 1);
			}
			if ($search_date_start) {
				$sql .= " AND f.datef >= '".$this->db->idate($search_date_start)."'";
			}
			if ($search_date_end) {
				$sql .= " AND f.datef <= '".$this->db->idate($search_date_end)."'";
			}
			if (strlen(trim($search_country))) {
				$arrayofcode = getCountriesInEEC();
				$country_code_in_EEC = $country_code_in_EEC_without_me = '';
				foreach ($arrayofcode as $key => $value) {
					$country_code_in_EEC .= ($country_code_in_EEC ? "," : "")."'".$value."'";
					if ($value != $mysoc->country_code) {
						$country_code_in_EEC_without_me .= ($country_code_in_EEC_without_me ? "," : "")."'".$value."'";
					}
				}
				if ($search_country == 'special_allnotme') {
					$sql .= " AND co.code <> '".$this->db->escape($mysoc->country_code)."'";
				} elseif ($search_country == 'special_eec') {
					$sql .= " AND co.code IN (".$this->db->sanitize($country_code_in_EEC, 1).")";
				} elseif ($search_country == 'special_eecnotme') {
					$sql .= " AND co.code IN (".$this->db->sanitize($country_code_in_EEC_without_me, 1).")";
				} elseif ($search_country == 'special_noteec') {
					$sql .= " AND co.code NOT IN (".$this->db->sanitize($country_code_in_EEC, 1).")";
				} else {
					$sql .= natural_search("co.code", $search_country);
				}
			}
			if (strlen(trim($search_tvaintra))) {
				$sql .= natural_search("s.tva_intra", $search_tvaintra);
			}
			if (!empty($conf->global->FACTURE_DEPOSITS_ARE_JUST_PAYMENTS)) {
				$sql .= " AND f.type IN (".Facture::TYPE_STANDARD.",".Facture::TYPE_REPLACEMENT.",".Facture::TYPE_CREDIT_NOTE.",".Facture::TYPE_SITUATION.")";
			} else {
				$sql .= " AND f.type IN (".Facture::TYPE_STANDARD.",".Facture::TYPE_REPLACEMENT.",".Facture::TYPE_CREDIT_NOTE.",".Facture::TYPE_DEPOSIT.",".Facture::TYPE_SITUATION.")";
			}
			$sql .= " AND f.entity IN (".getEntity('invoice', 0).")"; // We don't share object for accountancy

		}

        if (in_array('accountancysupplierlist', $context)) {
            global $sql, $chartaccountcode, $search_societe, $search_lineid, $search_invoice, $search_ref, $search_label, $search_desc, $search_amount, $search_account, $search_vat, $search_date_start, $search_date_end, $search_country, $search_tvaintra;

            $accountancy_code_sell_field = $this->db->ifsql("pac.accountancy_code_sell IS NULL OR pac.accountancy_code_sell = ''", 'p.accountancy_code_sell', 'pac.accountancy_code_sell');
            $accountancy_code_sell_intra_field = $this->db->ifsql("pac.accountancy_code_sell_intra IS NULL OR pac.accountancy_code_sell_intra = ''", 'p.accountancy_code_sell_intra', 'pac.accountancy_code_sell_intra');
            $accountancy_code_sell_export_field = $this->db->ifsql("pac.accountancy_code_sell_export IS NULL OR pac.accountancy_code_sell_export = ''", 'p.accountancy_code_sell_export', 'pac.accountancy_code_sell_export');
            $accountancy_code_buy_field = $this->db->ifsql("pac.accountancy_code_buy IS NULL OR pac.accountancy_code_buy = ''", 'p.accountancy_code_buy', 'pac.accountancy_code_buy');
            $accountancy_code_buy_intra_field = $this->db->ifsql("pac.accountancy_code_buy_intra IS NULL OR pac.accountancy_code_buy_intra = ''", 'p.accountancy_code_buy_intra', 'pac.accountancy_code_buy_intra');
            $accountancy_code_buy_export_field = $this->db->ifsql("pac.accountancy_code_buy_export IS NULL OR pac.accountancy_code_buy_export = ''", 'p.accountancy_code_buy_export', 'pac.accountancy_code_buy_export');

            // Supplier Invoice lines
            $sql = "SELECT f.rowid as facid, f.ref, f.datef, f.type as ftype,";
            $sql .= " l.rowid, l.fk_product, l.description, l.total_ht, l.fk_code_ventilation, l.product_type as type_l, l.tva_tx as tva_tx_line, l.vat_src_code,";
            $sql .= " p.rowid as product_id, p.ref as product_ref, p.label as product_label, p.fk_product_type as type, p.tva_tx as tva_tx_prod,";
            $sql .= " " . $accountancy_code_sell_field . " as code_sell, " . $accountancy_code_sell_intra_field . " as code_sell_intra, " . $accountancy_code_sell_export_field . " as code_sell_export,";
            $sql .= " " . $accountancy_code_buy_field . " as code_buy, " . $accountancy_code_buy_intra_field . " as code_buy_intra, " . $accountancy_code_buy_export_field . " as code_buy_export,";
            $sql .= " p.tosell as status, p.tobuy as status_buy,";
            $sql .= " aa.rowid as aarowid, aa2.rowid as aarowid_intra, aa3.rowid as aarowid_export, aa4.rowid as aarowid_thirdparty,";
            $sql .= " co.code as country_code, co.label as country_label,";
            $sql .= " s.rowid as socid, s.nom as name, s.tva_intra, s.email, s.town, s.zip, s.fk_pays, s.client, s.fournisseur, s.code_client, s.code_fournisseur, s.code_compta as code_compta_client, s.code_compta_fournisseur,";
            if (!empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED)) {
                $sql .= " spe.accountancy_code_buy as company_code_sell";
            } else {
                $sql .= " s.accountancy_code_buy as company_code_sell";
            }
            $sql .= " FROM ".MAIN_DB_PREFIX."facture_fourn as f";
            $sql .= " INNER JOIN ".MAIN_DB_PREFIX."societe as s ON s.rowid = f.fk_soc";
            if (!empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED)) {
                $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "societe_perentity as spe ON spe.fk_soc = s.rowid AND spe.entity = " . ((int) $conf->entity);
            }
            $sql .= " LEFT JOIN ".MAIN_DB_PREFIX."c_country as co ON co.rowid = s.fk_pays ";
            $sql .= " INNER JOIN ".MAIN_DB_PREFIX."facture_fourn_det as l ON f.rowid = l.fk_facture_fourn";
            $sql .= " LEFT JOIN ".MAIN_DB_PREFIX."product as p ON p.rowid = l.fk_product";
			$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "product_accountancy_code as pac ON pac.rowid = EPA_GET_SUPPLIER_JOIN_ID(l.rowid)";
            $sql .= " LEFT JOIN ".MAIN_DB_PREFIX."accounting_account as aa  ON " . $accountancy_code_buy_field . " = aa.account_number         AND aa.active = 1  AND aa.fk_pcg_version = '".$this->db->escape($chartaccountcode)."' AND aa.entity = ".$conf->entity;
            $sql .= " LEFT JOIN ".MAIN_DB_PREFIX."accounting_account as aa2 ON " . $accountancy_code_buy_intra_field . " = aa2.account_number  AND aa2.active = 1 AND aa2.fk_pcg_version = '".$this->db->escape($chartaccountcode)."' AND aa2.entity = ".$conf->entity;
            $sql .= " LEFT JOIN ".MAIN_DB_PREFIX."accounting_account as aa3 ON " . $accountancy_code_buy_export_field . " = aa3.account_number AND aa3.active = 1 AND aa3.fk_pcg_version = '".$this->db->escape($chartaccountcode)."' AND aa3.entity = ".$conf->entity;
            $alias_societe_perentity = empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED) ? "s" : "spe";
            $sql .= " LEFT JOIN ".MAIN_DB_PREFIX."accounting_account as aa4 ON " . $alias_societe_perentity . ".accountancy_code_buy = aa4.account_number        AND aa4.active = 1 AND aa4.fk_pcg_version = '".$this->db->escape($chartaccountcode)."' AND aa4.entity = ".$conf->entity;

            $sql .= " WHERE f.fk_statut > 0 AND l.fk_code_ventilation <= 0";
            $sql .= " AND l.product_type <= 2";
            // Define begin binding date
            if (!empty($conf->global->ACCOUNTING_DATE_START_BINDING)) {
                $sql .= " AND f.datef >= '".$this->db->idate($conf->global->ACCOUNTING_DATE_START_BINDING)."'";
            }
            // Add search filter like
            if ($search_societe) {
                $sql .= natural_search('s.nom', $search_societe);
            }
            if ($search_lineid) {
                $sql .= natural_search("l.rowid", $search_lineid, 1);
            }
            if (strlen(trim($search_invoice))) {
                $sql .= natural_search("f.ref", $search_invoice);
            }
            if (strlen(trim($search_ref))) {
                $sql .= natural_search("p.ref", $search_ref);
            }
            if (strlen(trim($search_label))) {
                $sql .= natural_search("p.label", $search_label);
            }
            if (strlen(trim($search_desc))) {
                $sql .= natural_search("l.description", $search_desc);
            }
            if (strlen(trim($search_amount))) {
                $sql .= natural_search("l.total_ht", $search_amount, 1);
            }
            if (strlen(trim($search_account))) {
                $sql .= natural_search("aa.account_number", $search_account);
            }
            if (strlen(trim($search_vat))) {
                $sql .= natural_search("l.tva_tx", price2num($search_vat), 1);
            }
            if ($search_date_start) {
                $sql .= " AND f.datef >= '".$this->db->idate($search_date_start)."'";
            }
            if ($search_date_end) {
                $sql .= " AND f.datef <= '".$this->db->idate($search_date_end)."'";
            }
            if (strlen(trim($search_country))) {
                $arrayofcode = getCountriesInEEC();
                $country_code_in_EEC = $country_code_in_EEC_without_me = '';
                foreach ($arrayofcode as $key => $value) {
                    $country_code_in_EEC .= ($country_code_in_EEC ? "," : "")."'".$value."'";
                    if ($value != $mysoc->country_code) {
                        $country_code_in_EEC_without_me .= ($country_code_in_EEC_without_me ? "," : "")."'".$value."'";
                    }
                }
                if ($search_country == 'special_allnotme') {
                    $sql .= " AND co.code <> '".$this->db->escape($mysoc->country_code)."'";
                } elseif ($search_country == 'special_eec') {
                    $sql .= " AND co.code IN (".$this->db->sanitize($country_code_in_EEC, 1).")";
                } elseif ($search_country == 'special_eecnotme') {
                    $sql .= " AND co.code IN (".$this->db->sanitize($country_code_in_EEC_without_me, 1).")";
                } elseif ($search_country == 'special_noteec') {
                    $sql .= " AND co.code NOT IN (".$this->db->sanitize($country_code_in_EEC, 1).")";
                } else {
                    $sql .= natural_search("co.code", $search_country);
                }
            }
            if (strlen(trim($search_tvaintra))) {
                $sql .= natural_search("s.tva_intra", $search_tvaintra);
            }
            if (!empty($conf->global->FACTURE_SUPPLIER_DEPOSITS_ARE_JUST_PAYMENTS)) {
                $sql .= " AND f.type IN (".FactureFournisseur::TYPE_STANDARD.",".FactureFournisseur::TYPE_REPLACEMENT.",".FactureFournisseur::TYPE_CREDIT_NOTE.",".FactureFournisseur::TYPE_SITUATION.")";
            } else {
                $sql .= " AND f.type IN (".FactureFournisseur::TYPE_STANDARD.",".FactureFournisseur::TYPE_REPLACEMENT.",".FactureFournisseur::TYPE_CREDIT_NOTE.",".FactureFournisseur::TYPE_DEPOSIT.",".FactureFournisseur::TYPE_SITUATION.")";
            }
            $sql .= " AND f.entity IN (".getEntity('facture_fourn', 0).")"; // We don't share object for accountancy

        }


		return 0;
	}

	/**
	 * Replace the processing of the action 'validate history' on accountancy/customer/index.php page
	 */
	function validateHistory()
	{
		$action = GETPOST('action', 'aZ09');
		if (!self::$not_validate_history && preg_match('/' . preg_quote('/accountancy/customer/index.php', '/') . '/', $_SERVER['PHP_SELF']) && $action == 'validatehistory') {
			self::$not_validate_history = true;
			global $langs, $conf, $mysoc, $user;

			$langs->loadLangs(array("compta", "bills", "other", "accountancy"));

			// Security check
			$user->getrights();
			if (empty($conf->accounting->enabled)) {
				accessforbidden();
			}
			if ($user->socid > 0) {
				accessforbidden();
			}
			if (!$user->rights->accounting->bind->write) {
				accessforbidden();
			}

			$chartaccountcode = dol_getIdFromCode($this->db, $conf->global->CHARTOFACCOUNTS, 'accounting_system', 'rowid', 'pcg_version');

			$error = 0;
			$this->db->begin();

			$accountancy_code_sell_field = $this->db->ifsql("pac.accountancy_code_sell IS NULL OR pac.accountancy_code_sell = ''", 'p.accountancy_code_sell', 'pac.accountancy_code_sell');
			$accountancy_code_sell_intra_field = $this->db->ifsql("pac.accountancy_code_sell_intra IS NULL OR pac.accountancy_code_sell_intra = ''", 'p.accountancy_code_sell_intra', 'pac.accountancy_code_sell_intra');
			$accountancy_code_sell_export_field = $this->db->ifsql("pac.accountancy_code_sell_export IS NULL OR pac.accountancy_code_sell_export = ''", 'p.accountancy_code_sell_export', 'pac.accountancy_code_sell_export');
			$accountancy_code_buy_field = $this->db->ifsql("pac.accountancy_code_buy IS NULL OR pac.accountancy_code_buy = ''", 'p.accountancy_code_buy', 'pac.accountancy_code_buy');
			$accountancy_code_buy_intra_field = $this->db->ifsql("pac.accountancy_code_buy_intra IS NULL OR pac.accountancy_code_buy_intra = ''", 'p.accountancy_code_buy_intra', 'pac.accountancy_code_buy_intra');
			$accountancy_code_buy_export_field = $this->db->ifsql("pac.accountancy_code_buy_export IS NULL OR pac.accountancy_code_buy_export = ''", 'p.accountancy_code_buy_export', 'pac.accountancy_code_buy_export');

			// Customer Invoice lines (must be same request than into page list.php for manual binding)
			$sql = "SELECT f.rowid as facid, f.ref as ref, f.datef, f.type as ftype,";
			$sql .= " l.rowid, l.fk_product, l.description, l.total_ht, l.fk_code_ventilation, l.product_type as type_l, l.tva_tx as tva_tx_line, l.vat_src_code,";
			$sql .= " p.rowid as product_id, p.ref as product_ref, p.label as product_label, p.fk_product_type as type, p.tva_tx as tva_tx_prod,";
			$sql .= " " . $accountancy_code_sell_field . " as code_sell, " . $accountancy_code_sell_intra_field . " as code_sell_intra, " . $accountancy_code_sell_export_field . " as code_sell_export,";
			$sql .= " " . $accountancy_code_buy_field . " as code_buy, " . $accountancy_code_buy_intra_field . " as code_buy_intra, " . $accountancy_code_buy_export_field . " as code_buy_export,";
			$sql .= " aa.rowid as aarowid, aa2.rowid as aarowid_intra, aa3.rowid as aarowid_export, aa4.rowid as aarowid_thirdparty,";
			$sql .= " co.code as country_code, co.label as country_label,";
			$sql .= " s.tva_intra,";
			if (!empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED)) {
				$sql .= " spe.accountancy_code_sell as company_code_sell";
			} else {
				$sql .= " s.accountancy_code_sell as company_code_sell";
			}

			$sql .= " FROM " . MAIN_DB_PREFIX . "facture as f";
			$sql .= " INNER JOIN " . MAIN_DB_PREFIX . "societe as s ON s.rowid = f.fk_soc";
			if (!empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED)) {
				$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "societe_perentity as spe ON spe.fk_soc = s.rowid AND spe.entity = " . ((int)$conf->entity);
			}
			$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "c_country as co ON co.rowid = s.fk_pays ";
			$sql .= " INNER JOIN " . MAIN_DB_PREFIX . "facturedet as l ON f.rowid = l.fk_facture";
			$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "product as p ON p.rowid = l.fk_product";
			$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "product_accountancy_code as pac ON pac.rowid = EPA_GET_CUSTOMER_JOIN_ID(l.rowid)";
			$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "accounting_account as aa  ON " . $accountancy_code_sell_field . " = aa.account_number         AND aa.active = 1  AND aa.fk_pcg_version = '" . $this->db->escape($chartaccountcode) . "' AND aa.entity = " . $conf->entity;
			$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "accounting_account as aa2 ON " . $accountancy_code_sell_intra_field . " = aa2.account_number  AND aa2.active = 1 AND aa2.fk_pcg_version = '" . $this->db->escape($chartaccountcode) . "' AND aa2.entity = " . $conf->entity;
			$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "accounting_account as aa3 ON " . $accountancy_code_sell_export_field . " = aa3.account_number AND aa3.active = 1 AND aa3.fk_pcg_version = '" . $this->db->escape($chartaccountcode) . "' AND aa3.entity = " . $conf->entity;
			$alias_societe_perentity = empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED) ? "s" : "spe";
			$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "accounting_account as aa4 ON " . $alias_societe_perentity . ".accountancy_code_sell = aa4.account_number        AND aa4.active = 1 AND aa4.fk_pcg_version = '" . $this->db->escape($chartaccountcode) . "' AND aa4.entity = " . $conf->entity;
			$sql .= " WHERE f.fk_statut > 0 AND l.fk_code_ventilation <= 0";
			$sql .= " AND l.product_type <= 2";
			$sql .= " AND f.entity IN (" . getEntity('invoice', 0) . ")"; // We don't share object for accountancy
			if (!empty($conf->global->ACCOUNTING_DATE_START_BINDING)) {
				$sql .= " AND f.datef >= '" . $this->db->idate($conf->global->ACCOUNTING_DATE_START_BINDING) . "'";
			}

			dol_syslog('htdocs/accountancy/customer/index.php');
			$result = $this->db->query($sql);
			if (!$result) {
				$error++;
				setEventMessages($this->db->lasterror(), null, 'errors');
			} else {
				$num_lines = $this->db->num_rows($result);

				require_once DOL_DOCUMENT_ROOT . '/compta/facture/class/facture.class.php';
				$facture_static = new Facture($this->db);

				require_once DOL_DOCUMENT_ROOT . '/core/lib/company.lib.php';
				$isSellerInEEC = isInEEC($mysoc);

				$i = 0;
				while ($i < min($num_lines, 10000)) {    // No more than 10000 at once
					$objp = $this->db->fetch_object($result);

					$isBuyerInEEC = isInEEC($objp);    // This make a database request but there is a cache into $conf->cache['country_code_in_EEC']

					// Level 2: Search suggested account for product/service (similar code exists in page list.php to make manual binding)
					$suggestedaccountingaccountfor = '';
					if (($objp->country_code == $mysoc->country_code) || empty($objp->country_code)) {  // If buyer in same country than seller (if not defined, we assume it is same country)
						$objp->code_sell_p = $objp->code_sell;
						$objp->aarowid_suggest = $objp->aarowid;
						$suggestedaccountingaccountfor = '';
					} else {
						if ($isSellerInEEC && $isBuyerInEEC && $objp->tva_tx_line != 0) {    // European intravat sale, but with VAT
							$objp->code_sell_p = $objp->code_sell;
							$objp->aarowid_suggest = $objp->aarowid;
							$suggestedaccountingaccountfor = 'eecwithvat';
						} elseif ($isSellerInEEC && $isBuyerInEEC && empty($objp->tva_intra)) {    // European intravat sale, without VAT intra community number
							$objp->code_sell_p = $objp->code_sell;
							$objp->aarowid_suggest = 0; // There is a doubt, no automatic binding
							$suggestedaccountingaccountfor = 'eecwithoutvatnumber';
						} elseif ($isSellerInEEC && $isBuyerInEEC) {          // European intravat sale
							$objp->code_sell_p = $objp->code_sell_intra;
							$objp->aarowid_suggest = $objp->aarowid_intra;
							$suggestedaccountingaccountfor = 'eec';
						} else {                                        // Foreign sale
							$objp->code_sell_p = $objp->code_sell_export;
							$objp->aarowid_suggest = $objp->aarowid_export;
							$suggestedaccountingaccountfor = 'export';
						}
					}

					if (!empty($conf->global->ACCOUNTANCY_USE_PRODUCT_ACCOUNT_ON_THIRDPARTY)) {
						// Level 3: Search suggested account for this thirdparty (similar code exists in page index.php to make automatic binding)
						if (!empty($objp->company_code_sell)) {
							$objp->code_sell_t = $objp->company_code_sell;
							$objp->aarowid_suggest = $objp->aarowid_thirdparty;
							$suggestedaccountingaccountfor = '';
						}
					}

					// Manage Deposit
					if (!empty($conf->global->ACCOUNTING_ACCOUNT_CUSTOMER_DEPOSIT)) {
						if ($objp->description == "(DEPOSIT)" || $objp->ftype == $facture_static::TYPE_DEPOSIT) {
							$accountdeposittoventilated = new AccountingAccount($this->db);
							$accountdeposittoventilated->fetch('', $conf->global->ACCOUNTING_ACCOUNT_CUSTOMER_DEPOSIT, 1);
							$objp->code_sell_l = $accountdeposittoventilated->ref;
							$objp->code_sell_p = '';
							$objp->code_sell_t = '';
							$objp->aarowid_suggest = $accountdeposittoventilated->rowid;
						}
					}

					if ($objp->aarowid_suggest > 0) {
						$sqlupdate = "UPDATE " . MAIN_DB_PREFIX . "facturedet";
						$sqlupdate .= " SET fk_code_ventilation = " . ((int)$objp->aarowid_suggest);
						$sqlupdate .= " WHERE fk_code_ventilation <= 0 AND product_type <= 2 AND rowid = " . ((int)$objp->rowid);

						$resqlupdate = $this->db->query($sqlupdate);
						if (!$resqlupdate) {
							$error++;
							setEventMessages($this->db->lasterror(), null, 'errors');
							break;
						}
					}

					$i++;
				}
			}

			if ($error) {
				$this->db->rollback();
			} else {
				$this->db->commit();
				setEventMessages($langs->trans('AutomaticBindingDone'), null, 'mesgs');
			}

			$_GET['action'] = '';
			$_POST['action'] = '';
		}

        if (!self::$not_validate_history && preg_match('/' . preg_quote('/accountancy/supplier/index.php', '/') . '/', $_SERVER['PHP_SELF']) && $action == 'validatehistory') {
            self::$not_validate_history = true;
            global $langs, $conf, $mysoc, $user;

            $langs->loadLangs(array("compta", "bills", "other", "accountancy"));

            // Security check
            $user->getrights();
            if (empty($conf->accounting->enabled)) {
                accessforbidden();
            }
            if ($user->socid > 0) {
                accessforbidden();
            }
            if (!$user->rights->accounting->bind->write) {
                accessforbidden();
            }

            $chartaccountcode = dol_getIdFromCode($this->db, $conf->global->CHARTOFACCOUNTS, 'accounting_system', 'rowid', 'pcg_version');

            $error = 0;
            $this->db->begin();

            $accountancy_code_sell_field = $this->db->ifsql("pac.accountancy_code_sell IS NULL OR pac.accountancy_code_sell = ''", 'p.accountancy_code_sell', 'pac.accountancy_code_sell');
            $accountancy_code_sell_intra_field = $this->db->ifsql("pac.accountancy_code_sell_intra IS NULL OR pac.accountancy_code_sell_intra = ''", 'p.accountancy_code_sell_intra', 'pac.accountancy_code_sell_intra');
            $accountancy_code_sell_export_field = $this->db->ifsql("pac.accountancy_code_sell_export IS NULL OR pac.accountancy_code_sell_export = ''", 'p.accountancy_code_sell_export', 'pac.accountancy_code_sell_export');
            $accountancy_code_buy_field = $this->db->ifsql("pac.accountancy_code_buy IS NULL OR pac.accountancy_code_buy = ''", 'p.accountancy_code_buy', 'pac.accountancy_code_buy');
            $accountancy_code_buy_intra_field = $this->db->ifsql("pac.accountancy_code_buy_intra IS NULL OR pac.accountancy_code_buy_intra = ''", 'p.accountancy_code_buy_intra', 'pac.accountancy_code_buy_intra');
            $accountancy_code_buy_export_field = $this->db->ifsql("pac.accountancy_code_buy_export IS NULL OR pac.accountancy_code_buy_export = ''", 'p.accountancy_code_buy_export', 'pac.accountancy_code_buy_export');

            // Customer Invoice lines (must be same request than into page list.php for manual binding)
            $sql = "SELECT f.rowid as facid, f.ref as ref, f.datef, f.type as ftype,";
            $sql .= " l.rowid, l.fk_product, l.description, l.total_ht, l.fk_code_ventilation, l.product_type as type_l, l.tva_tx as tva_tx_line, l.vat_src_code,";
            $sql .= " p.rowid as product_id, p.ref as product_ref, p.label as product_label, p.fk_product_type as type, p.tva_tx as tva_tx_prod,";
            $sql .= " " . $accountancy_code_sell_field . " as code_sell, " . $accountancy_code_sell_intra_field . " as code_sell_intra, " . $accountancy_code_sell_export_field . " as code_sell_export,";
            $sql .= " " . $accountancy_code_buy_field . " as code_buy, " . $accountancy_code_buy_intra_field . " as code_buy_intra, " . $accountancy_code_buy_export_field . " as code_buy_export,";
            $sql .= " aa.rowid as aarowid, aa2.rowid as aarowid_intra, aa3.rowid as aarowid_export, aa4.rowid as aarowid_thirdparty,";
            $sql .= " co.code as country_code, co.label as country_label,";
            $sql .= " s.tva_intra,";
            if (!empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED)) {
                $sql .= " spe.accountancy_code_buy as company_code_sell";
            } else {
                $sql .= " s.accountancy_code_buy as company_code_sell";
            }

            $sql .= " FROM " . MAIN_DB_PREFIX . "facture_fourn as f";
            $sql .= " INNER JOIN " . MAIN_DB_PREFIX . "societe as s ON s.rowid = f.fk_soc";
            if (!empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED)) {
                $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "societe_perentity as spe ON spe.fk_soc = s.rowid AND spe.entity = " . ((int)$conf->entity);
            }
            $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "c_country as co ON co.rowid = s.fk_pays ";
            $sql .= " INNER JOIN " . MAIN_DB_PREFIX . "facture_fourn_det as l ON f.rowid = l.fk_facture_fourn";
            $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "product as p ON p.rowid = l.fk_product";
			$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "product_accountancy_code as pac ON pac.rowid = EPA_GET_SUPPLIER_JOIN_ID(l.rowid)";
            $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "accounting_account as aa  ON " . $accountancy_code_buy_field . " = aa.account_number         AND aa.active = 1  AND aa.fk_pcg_version = '" . $this->db->escape($chartaccountcode) . "' AND aa.entity = " . $conf->entity;
            $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "accounting_account as aa2 ON " . $accountancy_code_buy_intra_field . " = aa2.account_number  AND aa2.active = 1 AND aa2.fk_pcg_version = '" . $this->db->escape($chartaccountcode) . "' AND aa2.entity = " . $conf->entity;
            $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "accounting_account as aa3 ON " . $accountancy_code_buy_export_field . " = aa3.account_number AND aa3.active = 1 AND aa3.fk_pcg_version = '" . $this->db->escape($chartaccountcode) . "' AND aa3.entity = " . $conf->entity;
            $alias_societe_perentity = empty($conf->global->MAIN_COMPANY_PERENTITY_SHARED) ? "s" : "spe";
            $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "accounting_account as aa4 ON " . $alias_societe_perentity . ".accountancy_code_buy = aa4.account_number        AND aa4.active = 1 AND aa4.fk_pcg_version = '" . $this->db->escape($chartaccountcode) . "' AND aa4.entity = " . $conf->entity;
            $sql .= " WHERE f.fk_statut > 0 AND l.fk_code_ventilation <= 0";
            $sql .= " AND l.product_type <= 2";
            $sql .= " AND f.entity IN (" . getEntity('facture_fourn', 0) . ")"; // We don't share object for accountancy
            if (!empty($conf->global->ACCOUNTING_DATE_START_BINDING)) {
                $sql .= " AND f.datef >= '" . $this->db->idate($conf->global->ACCOUNTING_DATE_START_BINDING) . "'";
            }

            dol_syslog('htdocs/accountancy/supplier/index.php');
            $result = $this->db->query($sql);
            if (!$result) {
                $error++;
                setEventMessages($this->db->lasterror(), null, 'errors');
            } else {
                $num_lines = $this->db->num_rows($result);

                require_once DOL_DOCUMENT_ROOT . '/fourn/class/fournisseur.facture.class.php';
                $facture_static = new FactureFournisseur($this->db);

                require_once DOL_DOCUMENT_ROOT . '/core/lib/company.lib.php';
                $isSellerInEEC = isInEEC($mysoc);

                $i = 0;
                while ($i < min($num_lines, 10000)) {    // No more than 10000 at once
                    $objp = $this->db->fetch_object($result);

                    $isBuyerInEEC = isInEEC($objp);    // This make a database request but there is a cache into $conf->cache['country_code_in_EEC']

                    // Level 2: Search suggested account for product/service (similar code exists in page list.php to make manual binding)
                    $suggestedaccountingaccountfor = '';
                    if (($objp->country_code == $mysoc->country_code) || empty($objp->country_code)) {  // If buyer in same country than seller (if not defined, we assume it is same country)
                        $objp->code_buy_p = $objp->code_buy;
                        $objp->aarowid_suggest = $objp->aarowid;
                        $suggestedaccountingaccountfor = '';
                    } else {
                        if ($isSellerInEEC && $isBuyerInEEC && $objp->tva_tx_line != 0) {    // European intravat sale, but with VAT
                            $objp->code_buy_p = $objp->code_buy;
                            $objp->aarowid_suggest = $objp->aarowid;
                            $suggestedaccountingaccountfor = 'eecwithvat';
                        } elseif ($isSellerInEEC && $isBuyerInEEC && empty($objp->tva_intra)) {    // European intravat sale, without VAT intra community number
                            $objp->code_buy_p = $objp->code_buy;
                            $objp->aarowid_suggest = 0; // There is a doubt, no automatic binding
                            $suggestedaccountingaccountfor = 'eecwithoutvatnumber';
                        } elseif ($isSellerInEEC && $isBuyerInEEC) {          // European intravat sale
                            $objp->code_buy_p = $objp->code_buy_intra;
                            $objp->aarowid_suggest = $objp->aarowid_intra;
                            $suggestedaccountingaccountfor = 'eec';
                        } else {                                        // Foreign sale
                            $objp->code_buy_p = $objp->code_buy_export;
                            $objp->aarowid_suggest = $objp->aarowid_export;
                            $suggestedaccountingaccountfor = 'export';
                        }
                    }

                    if (!empty($conf->global->ACCOUNTANCY_USE_PRODUCT_ACCOUNT_ON_THIRDPARTY)) {
                        // Level 3: Search suggested account for this thirdparty (similar code exists in page index.php to make automatic binding)
                        if (!empty($objp->company_code_buy)) {
                            $objp->code_buy_t = $objp->company_code_buy;
                            $objp->aarowid_suggest = $objp->aarowid_thirdparty;
                            $suggestedaccountingaccountfor = '';
                        }
                    }

                    // Manage Deposit
                    if (!empty($conf->global->ACCOUNTING_ACCOUNT_SUPPLIER_DEPOSIT)) {
                        if ($objp->description == "(DEPOSIT)" || $objp->ftype == $facture_static::TYPE_DEPOSIT) {
                            $accountdeposittoventilated = new AccountingAccount($this->db);
                            $accountdeposittoventilated->fetch('', $conf->global->ACCOUNTING_ACCOUNT_CUSTOMER_DEPOSIT, 1);
                            $objp->code_buy_l = $accountdeposittoventilated->ref;
                            $objp->code_buy_p = '';
                            $objp->code_buy_t = '';
                            $objp->aarowid_suggest = $accountdeposittoventilated->rowid;
                        }
                    }

                    if ($objp->aarowid_suggest > 0) {
                        $sqlupdate = "UPDATE " . MAIN_DB_PREFIX . "facture_fourn_det";
                        $sqlupdate .= " SET fk_code_ventilation = " . ((int)$objp->aarowid_suggest);
                        $sqlupdate .= " WHERE fk_code_ventilation <= 0 AND product_type <= 2 AND rowid = " . ((int)$objp->rowid);

                        $resqlupdate = $this->db->query($sqlupdate);
                        if (!$resqlupdate) {
                            $error++;
                            setEventMessages($this->db->lasterror(), null, 'errors');
                            break;
                        }
                    }

                    $i++;
                }
            }

            if ($error) {
                $this->db->rollback();
            } else {
                $this->db->commit();
                setEventMessages($langs->trans('AutomaticBindingDone'), null, 'mesgs');
            }

            $_GET['action'] = '';
            $_POST['action'] = '';
        }
	}
}