<?php
/* Copyright (C) 2023  Open-Dsi         <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

/**
 *    \file       htdocs/product/class/productaccountancycode.class.php
 *    \ingroup    produit
 *    \brief      File of class to manage accountancy code for products or services
 */
require_once DOL_DOCUMENT_ROOT.'/core/class/commonobject.class.php';

/**
 * Class to manage accountancy code for products or services
 */
class ProductAccountancyCode extends CommonObject
{
	/**
	 * @var string ID to identify managed object
	 */
	public $element = 'productaccountancycode';

	/**
	 * @var string Name of table without prefix where object is stored
	 */
	public $table_element = 'product_accountancy_code';

	/**
	 * @var string Field with ID of parent key if this field has a parent
	 */
	public $fk_element = 'fk_product';

	/**
	 * @var array	List of child tables. To test if we can delete object.
	 */
	protected $childtables = array(
	);

	/**
	 * 0=No test on entity, 1=Test with field entity, 2=Test with link by societe
	 *
	 * @var int
	 */
	public $ismultientitymanaged = 1;

	/**
	 * @var string picto
	 */
	public $picto = 'product';

	/**
	 *  'type' if the field format ('integer', 'integer:ObjectClass:PathToClass[:AddCreateButtonOrNot[:Filter]]', 'varchar(x)', 'double(24,8)', 'real', 'price', 'text', 'html', 'date', 'datetime', 'timestamp', 'duration', 'mail', 'phone', 'url', 'password')
	 *         Note: Filter can be a string like "(t.ref:like:'SO-%') or (t.date_creation:<:'20160101') or (t.nature:is:NULL)"
	 *  'label' the translation key.
	 *  'enabled' is a condition when the field must be managed (Example: 1 or '$conf->global->MY_SETUP_PARAM)
	 *  'position' is the sort order of field.
	 *  'notnull' is set to 1 if not null in database. Set to -1 if we must set data to null if empty ('' or 0).
	 *  'visible' says if field is visible in list (Examples: 0=Not visible, 1=Visible on list and create/update/view forms, 2=Visible on list only, 3=Visible on create/update/view form only (not list), 4=Visible on list and update/view form only (not create). 5=Visible on list and view only (not create/not update). Using a negative value means field is not shown by default on list but can be selected for viewing)
	 *  'noteditable' says if field is not editable (1 or 0)
	 *  'default' is a default value for creation (can still be overwrote by the Setup of Default Values if field is editable in creation form). Note: If default is set to '(PROV)' and field is 'ref', the default value will be set to '(PROVid)' where id is rowid when a new record is created.
	 *  'index' if we want an index in database.
	 *  'foreignkey'=>'tablename.field' if the field is a foreign key (it is recommanded to name the field fk_...).
	 *  'searchall' is 1 if we want to search in this field when making a search from the quick search button.
	 *  'isameasure' must be set to 1 if you want to have a total on list for this field. Field type must be summable like integer or double(24,8).
	 *  'css' is the CSS style to use on field. For example: 'maxwidth200'
	 *  'help' is a string visible as a tooltip on field
	 *  'showoncombobox' if value of the field must be visible into the label of the combobox that list record
	 *  'disabled' is 1 if we want to have the field locked by a 'disabled' attribute. In most cases, this is never set into the definition of $fields into class, but is set dynamically by some part of code.
	 *  'arrayofkeyval' to set list of value if type is a list of predefined values. For example: array("0"=>"Draft","1"=>"Active","-1"=>"Cancel")
	 *  'autofocusoncreate' to have field having the focus on a create form. Only 1 field should have this property set to 1.
	 *  'comment' is not used. You can store here any text of your choice. It is not used by application.
	 *
	 *  Note: To have value dynamic, you can set value to 0 in definition and edit the value on the fly into the constructor.
	 */

	/**
	 * @var array fields of object product
	 */
//	public $fields = array(
//		'rowid' => array('type'=>'integer', 'label'=>'TechnicalID', 'enabled'=>1, 'visible'=>-2, 'notnull'=>1, 'index'=>1, 'position'=>1, 'comment'=>'Id'),
//		'ref'           =>array('type'=>'varchar(128)', 'label'=>'Ref', 'enabled'=>1, 'visible'=>1, 'notnull'=>1, 'showoncombobox'=>1, 'index'=>1, 'position'=>10, 'searchall'=>1, 'comment'=>'Reference of object'),
//		'entity'        =>array('type'=>'integer', 'label'=>'Entity', 'enabled'=>1, 'visible'=>0, 'default'=>1, 'notnull'=>1, 'index'=>1, 'position'=>5),
//		'label'         =>array('type'=>'varchar(255)', 'label'=>'Label', 'enabled'=>1, 'visible'=>1, 'notnull'=>1, 'showoncombobox'=>2, 'position'=>15),
//		'barcode'       =>array('type'=>'varchar(255)', 'label'=>'Barcode', 'enabled'=>'!empty($conf->barcode->enabled)', 'position'=>20, 'visible'=>-1, 'showoncombobox'=>3),
//		'fk_barcode_type' => array('type'=>'integer', 'label'=>'BarcodeType', 'enabled'=>'1', 'position'=>21, 'notnull'=>0, 'visible'=>-1,),
//		'note_public'   =>array('type'=>'html', 'label'=>'NotePublic', 'enabled'=>1, 'visible'=>0, 'position'=>61),
//		'note'          =>array('type'=>'html', 'label'=>'NotePrivate', 'enabled'=>1, 'visible'=>0, 'position'=>62),
//		'datec'         =>array('type'=>'datetime', 'label'=>'DateCreation', 'enabled'=>1, 'visible'=>-2, 'notnull'=>1, 'position'=>500),
//		'tms'           =>array('type'=>'timestamp', 'label'=>'DateModification', 'enabled'=>1, 'visible'=>-2, 'notnull'=>1, 'position'=>501),
//		//'date_valid'    =>array('type'=>'datetime',     'label'=>'DateCreation',     'enabled'=>1, 'visible'=>-2, 'position'=>502),
//		'fk_user_author'=>array('type'=>'integer', 'label'=>'UserAuthor', 'enabled'=>1, 'visible'=>-2, 'notnull'=>1, 'position'=>510, 'foreignkey'=>'llx_user.rowid'),
//		'fk_user_modif' =>array('type'=>'integer', 'label'=>'UserModif', 'enabled'=>1, 'visible'=>-2, 'notnull'=>-1, 'position'=>511),
//		//'fk_user_valid' =>array('type'=>'integer',      'label'=>'UserValidation',        'enabled'=>1, 'visible'=>-1, 'position'=>512),
//		'localtax1_tx' => array('type'=>'double(6,3)', 'label'=>'Localtax1tx', 'enabled'=>'1', 'position'=>150, 'notnull'=>0, 'visible'=>-1,),
//		'localtax1_type' => array('type'=>'varchar(10)', 'label'=>'Localtax1type', 'enabled'=>'1', 'position'=>155, 'notnull'=>1, 'visible'=>-1,),
//		'localtax2_tx' => array('type'=>'double(6,3)', 'label'=>'Localtax2tx', 'enabled'=>'1', 'position'=>160, 'notnull'=>0, 'visible'=>-1,),
//		'localtax2_type' => array('type'=>'varchar(10)', 'label'=>'Localtax2type', 'enabled'=>'1', 'position'=>165, 'notnull'=>1, 'visible'=>-1,),
//		'import_key'    =>array('type'=>'varchar(14)', 'label'=>'ImportId', 'enabled'=>1, 'visible'=>-2, 'notnull'=>-1, 'index'=>0, 'position'=>1000),
//		//'tosell'       =>array('type'=>'integer',      'label'=>'Status',           'enabled'=>1, 'visible'=>1,  'notnull'=>1, 'default'=>0, 'index'=>1,  'position'=>1000, 'arrayofkeyval'=>array(0=>'Draft', 1=>'Active', -1=>'Cancel')),
//		//'tobuy'        =>array('type'=>'integer',      'label'=>'Status',           'enabled'=>1, 'visible'=>1,  'notnull'=>1, 'default'=>0, 'index'=>1,  'position'=>1000, 'arrayofkeyval'=>array(0=>'Draft', 1=>'Active', -1=>'Cancel')),
//	);
	public $fk_product;
    public $product_type;
	public $free_line;
	public $vat_code;
	public $tva_tx;

	public $key_values;

	public $accountancy_code_sell;
	public $accountancy_code_sell_intra;
	public $accountancy_code_sell_export;
	public $accountancy_code_buy;
	public $accountancy_code_buy_intra;
	public $accountancy_code_buy_export;

	/**
	 * @var string[] 	List of accountancy code label cached
	 * @see $this->getAccountancyCodeLabel();
	 */
	public static $accountancy_code_label_cached = array();

	/**
	 * @var array[string] 	List of key type info
	 * @see $this->initialize();
	 */
	public static $key_type_info_list = null;

	// Key types
	const KEY_TYPE_THIRD_PARTY_TYPE = 'epa_third_party_type';

	/**
	 *  Constructor
	 *
	 * @param DoliDB $db Database handler
	 */
	public function __construct($db)
	{
		$this->db = $db;
		$this->initialize();
	}

	/**
	 * Initialize data
	 *
	 * @param boolean	$force		Force reload
	 */
	public function initialize($force = false)
	{
		global $hookmanager;

		if (!isset(self::$key_type_info_list) || $force) {
			self::$key_type_info_list = array(
				// For key on third party type
				self::KEY_TYPE_THIRD_PARTY_TYPE => array(
					'type' => self::KEY_TYPE_THIRD_PARTY_TYPE,
					'label' => "ThirdPartyType",
					'lang_files' => array('companies'),
					// Used in fetch function
					'keys_fields' => array('epa_thirdparty_type'),
					'keys_fields_dates' => array(),
					// Used in the list page
					'list_colspan' => 1,
					'list_sql_select' => 'ct.libelle',
					'list_sql_from' => ' LEFT JOIN ' . MAIN_DB_PREFIX . 'c_typent AS ct ON ct.id = pac.epa_thirdparty_type',
					// Used in the accountancy customer index and list page
					'sql_account_customer_from' => ' LEFT JOIN ' . MAIN_DB_PREFIX . 'societe as s ON s.rowid = f.fk_soc ',
					'sql_account_customer_join_on' => ' AND s.fk_typent = pac.epa_thirdparty_type',
					// Used in the accountancy supplier index and list page
					'sql_account_supplier_from' => ' LEFT JOIN ' . MAIN_DB_PREFIX . 'societe as s ON s.rowid = f.fk_soc ',
					'sql_account_supplier_join_on' => ' AND s.fk_typent = pac.epa_thirdparty_type',
				),
			);

			if (!isset($hookmanager)) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
				$hookmanager2 = new HookManager($this->db);
			} else {
				$hookmanager2 = clone $hookmanager;
			}

			$hookmanager2->initHooks(array('extendedproductaccountancydao'));
			$parameters = array();
			$reshook = $hookmanager2->executeHooks('epaInitialize', $parameters, $this); // Note that $action and $object may have been modified by some hooks

			// Fix old version
			foreach (self::$key_type_info_list as $k => $v) {
				if (!isset($v['sql_account_customer_from'])) self::$key_type_info_list[$k]['sql_account_customer_from'] = isset($v['sql_account_from']) ? $v['sql_account_from'] : '';
				if (!isset($v['sql_account_customer_join_on'])) self::$key_type_info_list[$k]['sql_account_customer_join_on'] = isset($v['sql_account_join_on']) ? $v['sql_account_join_on'] : '';
				if (!isset($v['sql_account_supplier_from'])) self::$key_type_info_list[$k]['sql_account_supplier_from'] = '';
				if (!isset($v['sql_account_supplier_join_on'])) self::$key_type_info_list[$k]['sql_account_supplier_join_on'] = '';
			}
		}
	}

	/**
	 * Create key values columns
	 *
	 * @return int				<0 if KO, >0 if OK
	 */
	public function createKeyValueColumns()
	{
		global $hookmanager;

		dol_syslog(__METHOD__);

		$error = 0;
		$this->db->begin();

		$key_conf = $this->getKeyConf();
		if (!empty($key_conf)) {
			if (!isset($hookmanager)) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
				$hookmanager2 = new HookManager($this->db);
			} else {
				$hookmanager2 = clone $hookmanager;
			}

			$hookmanager2->initHooks(array('extendedproductaccountancydao'));
			$parameters = array();
			$reshook = $hookmanager2->executeHooks('epaCreateKeyValueColumns', $parameters, $this); // Note that $action and $object may have been modified by some hooks
			if ($reshook < 0) {
				setEventMessages($hookmanager2->error, $hookmanager2->errors, 'errors');
				$error++;
			} elseif (empty($reshook)) {
				switch ($key_conf['type']) {
					case self::KEY_TYPE_THIRD_PARTY_TYPE:
						$sql = "ALTER TABLE " . MAIN_DB_PREFIX . "product_accountancy_code ADD COLUMN epa_thirdparty_type integer NOT NULL";
						$resql = $this->db->query($sql);
						if (!$resql && $this->db->lasterrno() != "DB_ERROR_COLUMN_ALREADY_EXISTS") {
							$this->errors[] = $this->db->lasterror();
							$error++;
						}

						break;
				}
			}
		}

		if (!$error) {
			$result = $this->createProcedures();
			if ($result < 0) {
				$error++;
			}
		}

		if ($error) {
			$this->db->rollback();
			return -1;
		} else {
			$this->db->commit();
			return 1;
		}
	}

	/**
	 * Update procedure for SQL statement join
	 *
	 * @return int				<0 if KO, >0 if OK
	 */
	public function createProcedures()
	{
		global $hookmanager;

		dol_syslog(__METHOD__);

		$error = 0;
		$this->db->begin();

		$resql = $this->db->query("DROP FUNCTION IF EXISTS EPA_GET_CUSTOMER_JOIN_ID");
		if (!$resql) {
			$this->errors[] = $this->db->lasterror();
			$error++;
		}

		if (!$error) {
			$resql = $this->db->query("DROP FUNCTION IF EXISTS EPA_GET_SUPPLIER_JOIN_ID");
			if (!$resql) {
				$this->errors[] = $this->db->lasterror();
				$error++;
			}
		}

		if (!$error) {
			$key_conf = $this->getKeyConf();
			if (!isset($hookmanager)) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
				$hookmanager2 = new HookManager($this->db);
			} else {
				$hookmanager2 = clone $hookmanager;
			}

			$hookmanager2->initHooks(array('extendedproductaccountancydao'));
			$parameters = array();
			$reshook = $hookmanager2->executeHooks('epaCreateProcedures', $parameters, $this); // Note that $action and $object may have been modified by some hooks
			if ($reshook < 0) {
				setEventMessages($hookmanager2->error, $hookmanager2->errors, 'errors');
				$error++;
			} elseif (empty($reshook)) {
				$sql = "CREATE FUNCTION EPA_GET_CUSTOMER_JOIN_ID(invoice_id INT)\n";
				$sql .= "RETURNS INT\n";
				$sql .= "DETERMINISTIC\n";
				$sql .= "BEGIN\n";
				$sql .= " DECLARE product_accountancy_code_id INT;\n";
				// Search pac.fk_product = p.rowid match
				$sql .= " SET product_accountancy_code_id = (\n";
				$sql .= "  SELECT pac.rowid AS product_accountancy_code_id";
				$sql .= "  FROM " . MAIN_DB_PREFIX . "facturedet as l";
				$sql .= "  LEFT JOIN " . MAIN_DB_PREFIX . "facture as f ON f.rowid = l.fk_facture";
				$sql .= "  LEFT JOIN " . MAIN_DB_PREFIX . "product as p ON p.rowid = l.fk_product";
				if (!empty($key_conf) && !empty($key_conf['sql_account_customer_from'])) $sql .= $key_conf['sql_account_customer_from'];
				$sql .= "  LEFT JOIN " . MAIN_DB_PREFIX . "product_accountancy_code as pac ON pac.fk_product = p.rowid AND pac.vat_code = " . $this->db->ifsql("l.vat_src_code IS NULL", "''", 'l.vat_src_code') . " AND pac.tva_tx = l.tva_tx AND pac.entity = " . (empty($conf->global->MAIN_PRODUCT_PERENTITY_SHARED) ? 'p.entity' : ((int)$conf->entity));
				if (!empty($key_conf) && !empty($key_conf['sql_account_customer_join_on'])) $sql .= $key_conf['sql_account_customer_join_on'];
				$sql .= "  WHERE l.rowid = invoice_id";
				$sql .= "  LIMIT 1);\n";
				// Search all and not free line
				$sql .= " IF (product_accountancy_code_id IS NULL OR product_accountancy_code_id = 0) THEN\n";
				$sql .= "   SET product_accountancy_code_id = (\n";
				$sql .= "    SELECT pac.rowid AS product_accountancy_code_id";
				$sql .= "    FROM " . MAIN_DB_PREFIX . "facturedet as l";
				$sql .= "    LEFT JOIN " . MAIN_DB_PREFIX . "facture as f ON f.rowid = l.fk_facture";
				$sql .= "    LEFT JOIN " . MAIN_DB_PREFIX . "product as p ON p.rowid = l.fk_product";
				if (!empty($key_conf) && !empty($key_conf['sql_account_customer_from'])) $sql .= $key_conf['sql_account_customer_from'];
				$sql .= "    LEFT JOIN " . MAIN_DB_PREFIX . "product_accountancy_code as pac ON pac.fk_product = 0 AND (pac.product_type = 10 OR pac.product_type = p.fk_product_type) AND pac.free_line IS NULL AND p.rowid IS NOT NULL AND pac.vat_code = " . $this->db->ifsql("l.vat_src_code IS NULL", "''", 'l.vat_src_code') . " AND pac.tva_tx = l.tva_tx AND pac.entity = " . (empty($conf->global->MAIN_PRODUCT_PERENTITY_SHARED) ? 'p.entity' : ((int)$conf->entity));
				if (!empty($key_conf) && !empty($key_conf['sql_account_customer_join_on'])) $sql .= $key_conf['sql_account_customer_join_on'];
				$sql .= "    WHERE l.rowid = invoice_id";
				$sql .= "    LIMIT 1);\n";
				$sql .= " END IF;\n";
				// Search all and free line
				$sql .= " IF (product_accountancy_code_id IS NULL OR product_accountancy_code_id = 0) THEN\n";
				$sql .= "   SET product_accountancy_code_id = (\n";
				$sql .= "    SELECT pac.rowid AS product_accountancy_code_id";
				$sql .= "    FROM " . MAIN_DB_PREFIX . "facturedet as l";
				$sql .= "    LEFT JOIN " . MAIN_DB_PREFIX . "facture as f ON f.rowid = l.fk_facture";
				if (!empty($key_conf) && !empty($key_conf['sql_account_customer_from'])) $sql .= $key_conf['sql_account_customer_from'];
				$sql .= "    LEFT JOIN " . MAIN_DB_PREFIX . "product_accountancy_code as pac ON pac.fk_product = 0 AND (pac.product_type = 10 OR pac.product_type = l.product_type) AND pac.free_line = 1 AND (l.fk_product IS NULL OR l.fk_product = 0) AND pac.vat_code = " . $this->db->ifsql("l.vat_src_code IS NULL", "''", 'l.vat_src_code') . " AND pac.tva_tx = l.tva_tx AND pac.entity = " . (empty($conf->global->MAIN_PRODUCT_PERENTITY_SHARED) ? 'f.entity' : ((int)$conf->entity));
				if (!empty($key_conf) && !empty($key_conf['sql_account_customer_join_on'])) $sql .= $key_conf['sql_account_customer_join_on'];
				$sql .= "    WHERE l.rowid = invoice_id";
				$sql .= "    LIMIT 1);\n";
				$sql .= " END IF;\n";
				$sql .= " RETURN product_accountancy_code_id;\n";
				$sql .= "END\n";

				$resql = $this->db->query($sql);
				if (!$resql) {
					$this->errors[] = $this->db->lasterror();
					$error++;
				}

				if (!$error) {
					$sql = "CREATE FUNCTION EPA_GET_SUPPLIER_JOIN_ID(supplier_invoice_id INT)\n";
					$sql .= "RETURNS INT\n";
					$sql .= "DETERMINISTIC\n";
					$sql .= "BEGIN\n";
					$sql .= " DECLARE product_accountancy_code_id INT;\n";
					// Search pac.fk_product = p.rowid match
					$sql .= " SET product_accountancy_code_id = (\n";
					$sql .= "  SELECT pac.rowid AS product_accountancy_code_id";
					$sql .= "  FROM " . MAIN_DB_PREFIX . "facture_fourn_det as l";
					$sql .= "  LEFT JOIN " . MAIN_DB_PREFIX . "facture_fourn as f ON f.rowid = l.fk_facture_fourn";
					$sql .= "  LEFT JOIN " . MAIN_DB_PREFIX . "product as p ON p.rowid = l.fk_product";
					if (!empty($key_conf) && !empty($key_conf['sql_account_supplier_from'])) $sql .= $key_conf['sql_account_supplier_from'];
					$sql .= "  LEFT JOIN " . MAIN_DB_PREFIX . "product_accountancy_code as pac ON (pac.fk_product = p.rowid OR (pac.fk_product = 0 AND (pac.product_type = 10 OR pac.product_type = p.fk_product_type) AND ((pac.free_line IS NULL AND p.rowid IS NOT NULL) OR (pac.free_line = 1 AND p.rowid IS NULL)))) AND pac.vat_code = " . $this->db->ifsql("l.vat_src_code IS NULL", "''", 'l.vat_src_code') . " AND pac.tva_tx = l.tva_tx AND pac.entity = " . (empty($conf->global->MAIN_PRODUCT_PERENTITY_SHARED) ? 'p.entity' : ((int)$conf->entity));
					if (!empty($key_conf) && !empty($key_conf['sql_account_supplier_join_on'])) $sql .= $key_conf['sql_account_supplier_join_on'];
					$sql .= "  WHERE l.rowid = supplier_invoice_id";
					$sql .= "  ORDER BY " . $this->db->ifsql('pac.fk_product = p.rowid', 0, $this->db->ifsql('pac.product_type = 10', 1, 2)) . " ASC";
					$sql .= "  LIMIT 1);\n";
					// Search all and not free line
					$sql .= " IF (product_accountancy_code_id IS NULL OR product_accountancy_code_id = 0) THEN\n";
					$sql .= "   SET product_accountancy_code_id = (\n";
					$sql .= "    SELECT pac.rowid AS product_accountancy_code_id";
					$sql .= "    FROM " . MAIN_DB_PREFIX . "facture_fourn_det as l";
					$sql .= "    LEFT JOIN " . MAIN_DB_PREFIX . "facture_fourn as f ON f.rowid = l.fk_facture_fourn";
					$sql .= "    LEFT JOIN " . MAIN_DB_PREFIX . "product as p ON p.rowid = l.fk_product";
					if (!empty($key_conf) && !empty($key_conf['sql_account_supplier_from'])) $sql .= $key_conf['sql_account_supplier_from'];
					$sql .= "    LEFT JOIN " . MAIN_DB_PREFIX . "product_accountancy_code as pac ON pac.fk_product = 0 AND (pac.product_type = 10 OR pac.product_type = p.fk_product_type) AND pac.free_line IS NULL AND p.rowid IS NOT NULL AND pac.vat_code = " . $this->db->ifsql("l.vat_src_code IS NULL", "''", 'l.vat_src_code') . " AND pac.tva_tx = l.tva_tx AND pac.entity = " . (empty($conf->global->MAIN_PRODUCT_PERENTITY_SHARED) ? 'p.entity' : ((int)$conf->entity));
					if (!empty($key_conf) && !empty($key_conf['sql_account_supplier_join_on'])) $sql .= $key_conf['sql_account_supplier_join_on'];
					$sql .= "    WHERE l.rowid = supplier_invoice_id";
					$sql .= "    LIMIT 1);\n";
					$sql .= " END IF;\n";
					// Search all and free line
					$sql .= " IF (product_accountancy_code_id IS NULL OR product_accountancy_code_id = 0) THEN\n";
					$sql .= "   SET product_accountancy_code_id = (\n";
					$sql .= "    SELECT pac.rowid AS product_accountancy_code_id";
					$sql .= "    FROM " . MAIN_DB_PREFIX . "facture_fourn_det as l";
					$sql .= "    LEFT JOIN " . MAIN_DB_PREFIX . "facture_fourn as f ON f.rowid = l.fk_facture_fourn";
					if (!empty($key_conf) && !empty($key_conf['sql_account_supplier_from'])) $sql .= $key_conf['sql_account_supplier_from'];
					$sql .= "    LEFT JOIN " . MAIN_DB_PREFIX . "product_accountancy_code as pac ON pac.fk_product = 0 AND (pac.product_type = 10 OR pac.product_type = l.product_type) AND pac.free_line = 1 AND (l.fk_product IS NULL OR l.fk_product = 0) AND pac.vat_code = " . $this->db->ifsql("l.vat_src_code IS NULL", "''", 'l.vat_src_code') . " AND pac.tva_tx = l.tva_tx AND pac.entity = " . (empty($conf->global->MAIN_PRODUCT_PERENTITY_SHARED) ? 'f.entity' : ((int)$conf->entity));
					if (!empty($key_conf) && !empty($key_conf['sql_account_supplier_join_on'])) $sql .= $key_conf['sql_account_supplier_join_on'];
					$sql .= "    WHERE l.rowid = supplier_invoice_id";
					$sql .= "    LIMIT 1);\n";
					$sql .= " END IF;\n";
					$sql .= " RETURN product_accountancy_code_id;\n";
					$sql .= "END\n";

					$resql = $this->db->query($sql);
					if (!$resql) {
						$this->errors[] = $this->db->lasterror();
						$error++;
					}
				}
			}
		}

		if ($error) {
			$this->db->rollback();
			return -1;
		} else {
			$this->db->commit();
			return 1;
		}
	}

	/**
	 *    Insert a product accountancy code into database
	 *
	 * @param  User $user      User making insert
	 * @param  int  $notrigger Disable triggers
	 * @return int             Id of product accountancy code if OK, < 0 if KO
	 */
	public function create($user, $notrigger = 0)
	{
		global $conf, $hookmanager, $langs;

		$error = 0;

		// Clean parameters
		$this->fk_product = $this->fk_product > 0 ? $this->fk_product : 0;
        $this->product_type = $this->fk_product > 0 ? -1 : (isset($this->product_type) && $this->product_type !== "" && $this->product_type >= 0 ? $this->product_type : 10);
		$this->free_line = !empty($this->free_line);
		$this->vat_code = trim($this->vat_code);
		$this->tva_tx = $this->tva_tx > 0 ? $this->tva_tx : 0;
		$this->key_values = is_array($this->key_values) ? $this->key_values : array();
		$this->entity = isset($this->entity) ? $this->entity : $conf->entity;
		$this->accountancy_code_buy = trim($this->accountancy_code_buy);
		$this->accountancy_code_buy_intra = trim($this->accountancy_code_buy_intra);
		$this->accountancy_code_buy_export = trim($this->accountancy_code_buy_export);
		$this->accountancy_code_sell = trim($this->accountancy_code_sell);
		$this->accountancy_code_sell_intra = trim($this->accountancy_code_sell_intra);
		$this->accountancy_code_sell_export = trim($this->accountancy_code_sell_export);

		// Check parameters
		if (empty($this->accountancy_code_buy) && empty($this->accountancy_code_buy_intra) && empty($this->accountancy_code_buy_export) &&
			empty($this->accountancy_code_sell) && empty($this->accountancy_code_sell_intra) && empty($this->accountancy_code_sell_export)
		) {
			$this->error = 'ErrorBadParameter';
			return -1;
		}

		// Check uniqueness
		$productaccountancycode = new ProductAccountancyCode($this->db);
		$result = $productaccountancycode->fetch('', $this->fk_product, $this->product_type, $this->free_line, $this->vat_code, $this->tva_tx, $this->key_values);
		if ($result < 0) {
			$this->errors[] = $productaccountancycode->errorsToString();
			return -1;
		} elseif ($result > 0) {
			$this->errors[] = $langs->trans("Error") . " : " . $langs->trans("ErrorProductAccountancyCodeAlreadyExists");
			return -1;
		}

		$this->db->begin();

		$sql = "INSERT INTO " . MAIN_DB_PREFIX . $this->table_element . " (";
		$sql .= "fk_product";
		$sql .= ", product_type";
		$sql .= ", free_line";
        $sql .= ", vat_code";
		$sql .= ", tva_tx";
		foreach ($this->key_values as $k => $v) {
			$sql .= ", " . $k;
		}
		$sql .= ", entity";
		$sql .= ", accountancy_code_buy";
		$sql .= ", accountancy_code_buy_intra";
		$sql .= ", accountancy_code_buy_export";
		$sql .= ", accountancy_code_sell";
		$sql .= ", accountancy_code_sell_intra";
		$sql .= ", accountancy_code_sell_export";
		$sql .= ")";
		$sql .= " VALUES (";
		$sql .= $this->fk_product;
		$sql .= ", " . ($this->product_type >= 0 ? $this->product_type : 'NULL');
		$sql .= ", " . (!empty($this->free_line) ? 1 : 'NULL');
        $sql .= ", '" . $this->db->escape($this->vat_code) . "'";
		$sql .= ", " . ((float)$this->tva_tx);
		foreach ($this->key_values as $k => $v) {
			$sql .= ", '" . $this->db->escape($v) . "'";
		}
		$sql .= ", " . ((int)$this->entity);
		$sql .= ", '" . $this->db->escape($this->accountancy_code_buy) . "'";
		$sql .= ", '" . $this->db->escape($this->accountancy_code_buy_intra) . "'";
		$sql .= ", '" . $this->db->escape($this->accountancy_code_buy_export) . "'";
		$sql .= ", '" . $this->db->escape($this->accountancy_code_sell) . "'";
		$sql .= ", '" . $this->db->escape($this->accountancy_code_sell_intra) . "'";
		$sql .= ", '" . $this->db->escape($this->accountancy_code_sell_export) . "'";
		$sql .= ")";

		dol_syslog(get_class($this) . "::Create", LOG_DEBUG);
		$result = $this->db->query($sql);
		if ($result) {
			$this->id = $this->db->last_insert_id(MAIN_DB_PREFIX . $this->table_element);

			if (!$error && !$notrigger) {
				// Call trigger
				$result = $this->call_trigger('PRODUCT_ACCOUNTANCY_CODE_CREATE', $user);
				if ($result < 0) {
					$error++;
				}
				// End call triggers
			}

			if (empty($error)) {
				$this->db->commit();
				return $this->id;
			} else {
				$this->db->rollback();
				return -1 * $error;
			}
		} else {
			$this->error = $langs->trans("Error") . " : " . $this->db->error() . " - " . $sql;
			$this->errors[] = $this->error;
			$this->db->rollback();
			return -1;
		}
	}

	/**
	 *    Update a product accountancy code into database
	 *
	 * @param  int     $id          Id of product
	 * @param  User    $user        Object user making update
	 * @param  int     $notrigger   Disable triggers
	 * @return int                  1 if OK, -1 if already exist, -2 if other error
	 */
	public function update($id, $user, $notrigger = false)
	{
		global $langs, $conf;

		$error = 0;

		// Clean parameters
		$id = $id > 0 ? $id : 0;
        $this->fk_product = $this->fk_product > 0 ? $this->fk_product : 0;
        $this->product_type = $this->fk_product > 0 ? -1 : (isset($this->product_type) && $this->product_type !== "" && $this->product_type >= 0 ? $this->product_type : 10);
		$this->free_line = !empty($this->free_line);
		$this->vat_code = trim($this->vat_code);
		$this->tva_tx = $this->tva_tx > 0 ? $this->tva_tx : 0;
		$this->key_values = is_array($this->key_values) ? $this->key_values : array();
		$this->accountancy_code_buy = trim($this->accountancy_code_buy);
		$this->accountancy_code_buy_intra = trim($this->accountancy_code_buy_intra);
		$this->accountancy_code_buy_export = trim($this->accountancy_code_buy_export);
		$this->accountancy_code_sell = trim($this->accountancy_code_sell);
		$this->accountancy_code_sell_intra = trim($this->accountancy_code_sell_intra);
		$this->accountancy_code_sell_export = trim($this->accountancy_code_sell_export);

		// Check parameters
		if (empty($this->accountancy_code_buy) && empty($this->accountancy_code_buy_intra) && empty($this->accountancy_code_buy_export) &&
			empty($this->accountancy_code_sell) && empty($this->accountancy_code_sell_intra) && empty($this->accountancy_code_sell_export)
		) {
			$this->error = 'ErrorBadParameter';
			return -1;
		}

		// Check uniqueness
		$productaccountancycode = new ProductAccountancyCode($this->db);
		$result = $productaccountancycode->fetch('', $this->fk_product, $this->product_type, $this->free_line, $this->vat_code, $this->tva_tx, $this->key_values);
		if ($result < 0) {
			$this->errors[] = $productaccountancycode->errorsToString();
			return -1;
		} elseif ($result > 0 && $productaccountancycode->id != $id) {
			$this->errors[] = $langs->trans("Error") . " : " . $langs->trans("ErrorProductAccountancyCodeAlreadyExists");
			return -1;
		}

		if (empty($this->oldcopy)) {
			$org = new self($this->db);
			$org->fetch($id);
			$this->oldcopy = $org;
		}

		$this->db->begin();
		$sql = "UPDATE " . MAIN_DB_PREFIX . $this->table_element;
		$sql .= " SET fk_product = " . $this->fk_product;
		$sql .= ", product_type = " . ($this->product_type >= 0 ? $this->product_type : 'NULL');
		$sql .= ", free_line = " . (!empty($this->free_line) ? 1 : 'NULL');
        $sql .= ", vat_code = '" . $this->db->escape($this->vat_code) . "'";
		$sql .= ", tva_tx = " . ((float)$this->tva_tx);

		foreach ($this->key_values as $k => $v) {
			$sql .= ", " . $k . " = '" . $this->db->escape($v) . "'";
		}

		$sql .= ", accountancy_code_buy = '" . $this->db->escape($this->accountancy_code_buy) . "'";
		$sql .= ", accountancy_code_buy_intra = '" . $this->db->escape($this->accountancy_code_buy_intra) . "'";
		$sql .= ", accountancy_code_buy_export = '" . $this->db->escape($this->accountancy_code_buy_export) . "'";
		$sql .= ", accountancy_code_sell= '" . $this->db->escape($this->accountancy_code_sell) . "'";
		$sql .= ", accountancy_code_sell_intra= '" . $this->db->escape($this->accountancy_code_sell_intra) . "'";
		$sql .= ", accountancy_code_sell_export= '" . $this->db->escape($this->accountancy_code_sell_export) . "'";

		// stock field is not here because it is a denormalized value from product_stock.
		$sql .= " WHERE rowid = " . ((int)$id);

		dol_syslog(get_class($this) . "::update", LOG_DEBUG);

		$resql = $this->db->query($sql);
		if ($resql) {
			$this->id = $id;

			if (!$error && !$notrigger) {
				// Call trigger
				$result = $this->call_trigger('PRODUCT_ACCOUNTANCY_CODE_MODIFY', $user);
				if ($result < 0) {
					$error++;
				}
				// End call triggers
			}

			if (!$error) {
				$this->db->commit();
				return 1;
			} else {
				$this->db->rollback();
				return -$error;
			}
		} else {
			if ($this->db->errno() == 'DB_ERROR_RECORD_ALREADY_EXISTS') {
				$langs->load("errors");
				$this->error = $langs->trans("Error") . " : " . $langs->trans("ErrorProductAccountancyCodeAlreadyExists");
				$this->errors[] = $this->error;
				$this->db->rollback();
				return -1;
			} else {
				$this->error = $langs->trans("Error") . " : " . $this->db->error() . " - " . $sql;
				$this->errors[] = $this->error;
				$this->db->rollback();
				return -2;
			}
		}
	}

	/**
	 *    Delete a product accountancy code from database
	 *
	 * @param  User $user      User (object) deleting product
	 * @param  int  $notrigger Do not execute trigger
	 * @return int             < 0 if KO, > 0 if OK
	 */
	public function delete(User $user, $notrigger = 0)
	{
		global $conf, $langs;

		$error = 0;

		// Check parameters
		if (empty($this->id)) {
			$this->error = "Object must be fetched before calling delete";
			return -1;
		}

		$this->db->begin();

		if (!$error && empty($notrigger)) {
			// Call trigger
			$result = $this->call_trigger('PRODUCT_ACCOUNTANCY_CODE_DELETE', $user);
			if ($result < 0) {
				$error++;
			}
			// End call triggers
		}

		// Delete product accountancy code
		if (!$error) {
			$sql = "DELETE FROM " . MAIN_DB_PREFIX . $this->table_element;
			$sql .= " WHERE rowid = " . (int)$this->id;

			$result = $this->db->query($sql);
			if (!$result) {
				$error++;
				$this->errors[] = $this->db->lasterror();
			}
		}

		if (!$error) {
			$this->db->commit();
			return 1;
		} else {
			foreach ($this->errors as $errmsg) {
				dol_syslog(get_class($this) . "::delete " . $errmsg, LOG_ERR);
				$this->error .= ($this->error ? ', ' . $errmsg : $errmsg);
			}
			$this->db->rollback();
			return -$error;
		}
	}

	/**
	 *  Load a product accountancy code in memory from database
	 *
	 * @param	int		$id					Id of product accountancy code to load
	 * @param	int		$fk_product			Id of product/service
     * @param	int		$product_type		Type of product/service
	 * @param	int		$free_line		Only free line
	 * @param	string	$vat_code			Vat code
	 * @param	int		$vat_rate			Vat rate
	 * @param	array	$key_values			List of Key value  array(field_name => value, ...)
	 * @return	int							<0 if KO, 0 if not found, >0 if OK
	 */
	public function fetch($id = '', $fk_product = 0, $product_type = -1, $free_line = 0, $vat_code = '', $vat_rate = 0, $key_values = array())
	{
		global $conf;

		dol_syslog(get_class($this) . "::fetch id=" . $id . " fk_product=" . $fk_product . " product_type=" . $product_type . " vat_code=" . $vat_code . " vat_rate=" . $vat_rate . " key_values=" . json_encode($key_values));

		// Clean parameters
		$id = $id > 0 ? $id : 0;
		$fk_product = $fk_product > 0 ? $fk_product : 0;
        $product_type = $fk_product > 0 ? -1 : (isset($product_type) && $product_type !== "" && $product_type >= 0 ? $product_type : 10);
		$free_line = !empty($free_line);
		$vat_code = trim($vat_code);
		$vat_rate = $vat_rate > 0 ? $vat_rate : 0;
		$key_values = is_array($key_values) ? $key_values : array();
		$key_conf = $this->getKeyConf();

		// Check parameters
		if (!$id && !empty($key_conf) && empty($key_values)) {
			$this->error = 'ErrorWrongParameters';
			dol_syslog(get_class($this) . "::fetch " . $this->error, LOG_ERR);
			return -1;
		}

		$sql = "SELECT t.rowid, t.fk_product, t.product_type, t.free_line, t.entity, t.vat_code, t.tva_tx";
		$sql .= (!empty($key_conf['keys_fields']) ? ", t." . implode(', t.', $key_conf['keys_fields']) : '');
		$sql .= ", t.accountancy_code_buy, t.accountancy_code_buy_intra, t.accountancy_code_buy_export, t.accountancy_code_sell, t.accountancy_code_sell_intra, t.accountancy_code_sell_export";
		$sql .= " FROM " . MAIN_DB_PREFIX . $this->table_element . ' AS t';
		if ($id) {
			$sql .= " WHERE t.rowid = " . ((int)$id);
		} else {
			if (empty($conf->global->MAIN_PRODUCT_PERENTITY_SHARED) && $fk_product > 0) {
				$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "product AS p ON p.rowid = t.fk_product";
				$sql .= " WHERE t.entity = p.entity";
			} else {
				$sql .= " WHERE t.entity = " . $conf->entity;
			}
			$sql .= " AND t.fk_product = " . ((int)$fk_product);
            $sql .= " AND t.product_type " . ($product_type >= 0 ? "= " . $product_type : "IS NULL");
			$sql .= " AND t.free_line " . (!empty($free_line) ? "= 1" : "IS NULL");
			$sql .= " AND t.vat_code = '" . $this->db->escape($vat_code) . "'";
			$sql .= " AND t.tva_tx = " . ((int)$vat_rate);
			foreach ($key_conf['keys_fields'] as $field_name) {
				$sql .= " AND t." . $field_name . " = '" . $this->db->escape($key_values[$field_name]) . "'";
			}
		}

		$resql = $this->db->query($sql);
		if ($resql) {
			unset($this->oldcopy);

			if ($this->db->num_rows($resql) > 0) {
				$obj = $this->db->fetch_object($resql);

				$this->id = $obj->rowid;
				$this->fk_product = $obj->fk_product;
				$this->product_type = $obj->product_type;
				$this->free_line = $obj->free_line;
				$this->entity = $obj->entity;

				$this->vat_code = $obj->vat_code;
				$this->tva_tx = $obj->tva_tx;

				if (!empty($key_conf['keys_fields'])) {
					foreach ($key_conf['keys_fields'] as $field_name) {
						$this->key_values[$field_name] = in_array($field_name, $key_conf['keys_fields_dates']) ? $this->db->idate($obj->$field_name) : $obj->$field_name;
					}
				}

				$this->accountancy_code_buy = $obj->accountancy_code_buy;
				$this->accountancy_code_buy_intra = $obj->accountancy_code_buy_intra;
				$this->accountancy_code_buy_export = $obj->accountancy_code_buy_export;
				$this->accountancy_code_sell = $obj->accountancy_code_sell;
				$this->accountancy_code_sell_intra = $obj->accountancy_code_sell_intra;
				$this->accountancy_code_sell_export = $obj->accountancy_code_sell_export;

				$this->db->free($resql);

				return 1;
			} else {
				return 0;
			}
		} else {
			$this->error = $this->db->lasterror();
			return -1;
		}
	}

	/**
	 * Get the key type list
	 *
	 * @return	array
	 */
	public function getKeyTypes()
	{
		global $langs;

		$key_types = array();
		if (is_array(self::$key_type_info_list)) {
			foreach (self::$key_type_info_list as $k => $info) {
				if (!empty($info['lang_files'])) $langs->loadLangs($info['lang_files']);
				$key_types[$k] = $langs->trans($info['label']);
			}
		}

		return $key_types;
	}

	/**
	 * Get the key configuration
	 *
	 * @return	array
	 */
	public function getKeyConf()
	{
		global $conf;

		$key_conf = array();
		if (isset(self::$key_type_info_list[getDolGlobalString('ACCOUNTING_PRODUCT_ACCOUNTANCY_CODE_ADDITIONAL_KEY')])) {
			$key_conf = self::$key_type_info_list[getDolGlobalString('ACCOUNTING_PRODUCT_ACCOUNTANCY_CODE_ADDITIONAL_KEY')];
		}

		return $key_conf;
	}

	/**
	 * Print the key values input (used for create / edit on the list)
	 */
	public function printKeyValuesListCreateEditInput()
	{
		global $conf, $hookmanager, $langs;

		dol_syslog(__METHOD__);

		$key_conf = $this->getKeyConf();
		if (!empty($key_conf)) {
			if (!isset($hookmanager)) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
				$hookmanager2 = new HookManager($this->db);
			} else {
				$hookmanager2 = clone $hookmanager;
			}

			$hookmanager2->initHooks(array('extendedproductaccountancydao'));
			$parameters = array();
			$reshook = $hookmanager2->executeHooks('epaPrintKeyValuesListCreateEditInput', $parameters, $this); // Note that $action and $object may have been modified by some hooks
			if ($reshook < 0) {
				setEventMessages($hookmanager2->error, $hookmanager2->errors, 'errors');
			} elseif (empty($reshook)) {
				switch ($key_conf['type']) {
					case self::KEY_TYPE_THIRD_PARTY_TYPE:
						global $form, $formcompany;

						if (!is_object($form)) {
							require_once DOL_DOCUMENT_ROOT . '/core/class/html.form.class.php';
							$form = new Form($this->db);
						}

						if (!is_object($formcompany)) {
							require_once DOL_DOCUMENT_ROOT . '/core/class/html.formcompany.class.php';
							$formcompany = new FormCompany($this->db);
						}

						print '<tr><td class="titlefieldcreate">' . $langs->trans($key_conf['label']) . '</td><td>';
						$epa_thirdparty_type = GETPOSTISSET('epa_thirdparty_type') ? GETPOST('epa_thirdparty_type', 'int') : $this->epa_thirdparty_type;
						$sortparam = (empty($conf->global->SOCIETE_SORT_ON_TYPEENT) ? 'ASC' : $conf->global->SOCIETE_SORT_ON_TYPEENT); // NONE means we keep sort of original array, so we sort on position. ASC, means next function will sort on label.
						print $form->selectarray("epa_thirdparty_type", $formcompany->typent_array(0), $epa_thirdparty_type, 1, 0, 0, '', 0, 0, 0, $sortparam, '', 1);
						print '</td></tr>';
						break;
				}
			}
		}
	}

	/**
	 * Get SQL used for filter the list
	 *
	 * @param	array	$search_key_values	List of search key values
	 * @return	string						SQL where statement
	 */
	public function getListSearchSQL($search_key_values)
	{
		global $hookmanager;

		dol_syslog(__METHOD__);

		$out = '';
		$key_conf = $this->getKeyConf();
		if (!empty($key_conf)) {
			if (!isset($hookmanager)) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
				$hookmanager2 = new HookManager($this->db);
			} else {
				$hookmanager2 = clone $hookmanager;
			}

			$hookmanager2->initHooks(array('extendedproductaccountancydao'));
			$parameters = array('search_key_values' => $search_key_values);
			$reshook = $hookmanager2->executeHooks('epaGetListSearchSQL', $parameters, $this); // Note that $action and $object may have been modified by some hooks
			if ($reshook < 0) {
				setEventMessages($hookmanager2->error, $hookmanager2->errors, 'errors');
			} elseif ($reshook > 0) {
				$out .= $hookmanager2->resPrint;
			} else {
				switch ($key_conf['type']) {
					case self::KEY_TYPE_THIRD_PARTY_TYPE:
						if (!empty($search_key_values['search_epa_thirdparty_type'])) {
							$out .= natural_search('ct.libelle', $search_key_values['search_epa_thirdparty_type']);
						}
						break;
				}
			}
		}

		return $out;
	}

	/**
	 * get search param key values
	 *
	 * @param	array	$search_key_values		List of search key values
	 * @return	string							The params value
	 */
	public function getKeyValuesListSearchParam($search_key_values)
	{
		global $hookmanager;

		dol_syslog(__METHOD__);

		$key_conf = $this->getKeyConf();
		$param = '';
		if (!empty($key_conf['type'])) {
			if (!isset($hookmanager)) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
				$hookmanager2 = new HookManager($this->db);
			} else {
				$hookmanager2 = clone $hookmanager;
			}

			$hookmanager2->initHooks(array('extendedproductaccountancydao'));
			$parameters = array('search_key_values' => $search_key_values);
			$reshook = $hookmanager2->executeHooks('epaGetKeyValuesListSearchParam', $parameters, $this); // Note that $action and $object may have been modified by some hooks
			if ($reshook < 0) {
				setEventMessages($hookmanager2->error, $hookmanager2->errors, 'errors');
			} elseif ($reshook > 0) {
				$param = $hookmanager2->resPrint;
			} else {
				switch ($key_conf['type']) {
					case self::KEY_TYPE_THIRD_PARTY_TYPE:
						if (!empty($search_key_values['search_epa_thirdparty_type'])) {
							$param .= "&search_epa_thirdparty_type=" . urlencode($search_key_values['search_epa_thirdparty_type']);
						}
						break;
				}
			}
		}

		return $param;
	}

	/**
	 * Print the key values search input (used for filter on the list)
	 *
	 * @param	array	$search_key_values	List of search key values
	 */
	public function printKeyValuesListSearch($search_key_values)
	{
		global $hookmanager;

		dol_syslog(__METHOD__);

		$key_conf = $this->getKeyConf();
		if (!empty($key_conf)) {
			if (!isset($hookmanager)) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
				$hookmanager2 = new HookManager($this->db);
			} else {
				$hookmanager2 = clone $hookmanager;
			}

			$hookmanager2->initHooks(array('extendedproductaccountancydao'));
			$parameters = array('search_key_values' => $search_key_values);
			$reshook = $hookmanager2->executeHooks('epaPrintKeyValuesListSearch', $parameters, $this); // Note that $action and $object may have been modified by some hooks
			if ($reshook < 0) {
				setEventMessages($hookmanager2->error, $hookmanager2->errors, 'errors');
			} elseif (empty($reshook)) {
				switch ($key_conf['type']) {
					case self::KEY_TYPE_THIRD_PARTY_TYPE:
						print '<td class="liste_titre left"><input class="flat" size="4" type="text" name="search_epa_thirdparty_type" value="'.$search_key_values['search_epa_thirdparty_type'].'"></td>';
						break;
				}
			}
		}
	}

	/**
	 * Print the key values title (used on the list)
	 *
	 *	@param	string	$moreparam   		Add more parameters on sort url links ("" by default)
	 *	@param  string	$sortfield   		Current field used to sort
	 *	@param  string	$sortorder   		Current sort order
	 */
	public function printKeyValuesListTitle($moreparam, $sortfield, $sortorder)
	{
		global $hookmanager, $langs;

		dol_syslog(__METHOD__);

		$key_conf = $this->getKeyConf();
		if (!empty($key_conf)) {
			if (!empty($key_conf['lang_files'])) $langs->loadLangs($key_conf['lang_files']);

			if (!isset($hookmanager)) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
				$hookmanager2 = new HookManager($this->db);
			} else {
				$hookmanager2 = clone $hookmanager;
			}

			$hookmanager2->initHooks(array('extendedproductaccountancydao'));
			$parameters = array('moreparam' => $moreparam, 'sortfield' => $sortfield, 'sortorder' => $sortorder);
			$reshook = $hookmanager2->executeHooks('epaPrintKeyValuesListTitle', $parameters, $this); // Note that $action and $object may have been modified by some hooks
			if ($reshook < 0) {
				setEventMessages($hookmanager2->error, $hookmanager2->errors, 'errors');
			} elseif (empty($reshook)) {
				switch ($key_conf['type']) {
					case self::KEY_TYPE_THIRD_PARTY_TYPE:
						print_liste_field_titre($langs->trans($key_conf['label']), $_SERVER["PHP_SELF"], 'ct.libelle', "", $moreparam, "", $sortfield, $sortorder);
						break;
				}
			}
		}
	}

	/**
	 * Print the key values title (used on the list)
	 *
	 *	@param	object	$obj   		SQL result line
	 */
	public function printKeyValuesListValue($obj)
	{
		global $hookmanager;

		dol_syslog(__METHOD__);

		$key_conf = $this->getKeyConf();
		if (!empty($key_conf)) {
			if (!isset($hookmanager)) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
				$hookmanager2 = new HookManager($this->db);
			} else {
				$hookmanager2 = clone $hookmanager;
			}

			$hookmanager2->initHooks(array('extendedproductaccountancydao'));
			$parameters = array('obj' => $obj);
			$reshook = $hookmanager2->executeHooks('epaPrintKeyValuesListValue', $parameters, $this); // Note that $action and $object may have been modified by some hooks
			if ($reshook < 0) {
				setEventMessages($hookmanager2->error, $hookmanager2->errors, 'errors');
			} elseif (empty($reshook)) {
				switch ($key_conf['type']) {
					case self::KEY_TYPE_THIRD_PARTY_TYPE:
						print '<td>';
						print $obj->libelle;
						print "</td>\n";
						break;
				}
			}
		}
	}

	/**
	 * get key values from POST
	 *
	 * @return	array|int												<0 if KO otherwise the list of key value
	 */
	public function getKeyValuesFromPost()
	{
		global $hookmanager, $langs;

		dol_syslog(__METHOD__);

		$key_conf = $this->getKeyConf();
		$key_values = array();
		if (!empty($key_conf['type'])) {
			if (!empty($key_conf['lang_files'])) $langs->loadLangs($key_conf['lang_files']);

			if (!isset($hookmanager)) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
				$hookmanager2 = new HookManager($this->db);
			} else {
				$hookmanager2 = clone $hookmanager;
			}

			$hookmanager2->initHooks(array('extendedproductaccountancydao'));
			$parameters = array();
			$reshook = $hookmanager2->executeHooks('epaGetKeyValuesFromPost', $parameters, $this); // Note that $action and $object may have been modified by some hooks
			if ($reshook < 0) {
				return -1;
			} elseif ($reshook > 0) {
				$key_values = $hookmanager2->resArray;
			} else {
				switch ($key_conf['type']) {
					case self::KEY_TYPE_THIRD_PARTY_TYPE:
						$epa_thirdparty_type = GETPOST('epa_thirdparty_type', 'int');
						if (!($epa_thirdparty_type > 0)) {
							setEventMessages($langs->trans('ErrorFieldRequired', $langs->transnoentities($key_conf['label'])), null, 'errors');
							return -1;
						} else {
							$key_values['epa_thirdparty_type'] = $epa_thirdparty_type;
						}
						break;
				}
			}
		}

		return $key_values;
	}

	/**
	 * get search key values from POST
	 *
	 * @return	array|int												<0 if KO otherwise the list of key value
	 */
	public function getSearchKeyValuesFromPost()
	{
		global $hookmanager, $langs;

		dol_syslog(__METHOD__);

		$key_conf = $this->getKeyConf();
		$search_key_values = array();
		if (!empty($key_conf['type'])) {
			if (!isset($hookmanager)) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
				$hookmanager2 = new HookManager($this->db);
			} else {
				$hookmanager2 = clone $hookmanager;
			}

			$hookmanager2->initHooks(array('extendedproductaccountancydao'));
			$parameters = array();
			$reshook = $hookmanager2->executeHooks('epaGetSearchKeyValuesFromPost', $parameters, $this); // Note that $action and $object may have been modified by some hooks
			if ($reshook < 0) {
				setEventMessages($hookmanager2->error, $hookmanager2->errors, 'errors');
			} elseif ($reshook > 0) {
				$search_key_values = $hookmanager2->resArray;
			} else {
				switch ($key_conf['type']) {
					case self::KEY_TYPE_THIRD_PARTY_TYPE:
						$search_epa_thirdparty_type = GETPOST('search_epa_thirdparty_type', 'alphanohtml');
						if (!empty($search_epa_thirdparty_type)) {
							$search_key_values['search_epa_thirdparty_type'] = $search_epa_thirdparty_type;
						}
						break;
				}
			}
		}

		return $search_key_values;
	}

	/**
	 * Add the accountancy code for a vat rate and the other key value
	 *
	 * @param	User			$user							User to set accountancy code
     * @param	int				$product_id						Product ID
	 * @param	int				$product_type					If product ID not provided (10: all, 0: product, 1: service)
	 * @param	int				$free_line					Only free line
	 * @param	string			$vat_code						VAT code
	 * @param	int				$vat_rate						VAT rate
	 * @param	array			$key_values						List of Key value  array(field_name => value, ...)
	 * @param	string			$accountancy_code_sell			Accountancy code sell
	 * @param	string			$accountancy_code_sell_intra	Accountancy code sell intra
	 * @param	string			$accountancy_code_sell_export	Accountancy code sell export
	 * @param	string			$accountancy_code_buy			Accountancy code buy
	 * @param	string			$accountancy_code_buy_intra		Accountancy code buy intra
	 * @param	string			$accountancy_code_buy_export	Accountancy code buy export
	 * @param	int				$notrigger						Do not execute trigger
	 * @return	int												<0 if KO, >0 if OK
	 */
	public function addAccountancyCode(User $user, $product_id, $product_type, $free_line, $vat_code, $vat_rate, $key_values = array(), $accountancy_code_sell = '', $accountancy_code_sell_intra = '', $accountancy_code_sell_export = '', $accountancy_code_buy = '', $accountancy_code_buy_intra = '', $accountancy_code_buy_export = '', $notrigger = 0)
	{
		global $conf, $langs;

		dol_syslog(__METHOD__ . " userid=$user->id, productid=$product_id, product_type=$product_type, free_line=$free_line, vat_code=$vat_code, vat_rate=$vat_rate" . ", key_values=" . json_encode($key_values) . ", accountancy_code_sell=$accountancy_code_sell, accountancy_code_sell_intra=$accountancy_code_sell_intra, accountancy_code_sell_export=$accountancy_code_sell_export, accountancy_code_buy=$accountancy_code_buy, accountancy_code_buy_intra=$accountancy_code_buy_intra, accountancy_code_buy_export=$accountancy_code_buy_export, notrigger=$notrigger");

		// Add accountancy code
		$productaccountancycode = new ProductAccountancyCode($this->db);

		$productaccountancycode->context = $this->context;
        $productaccountancycode->fk_product = $product_id;
        $productaccountancycode->product_type = $product_type;
		$productaccountancycode->free_line = $free_line;
		// Todo c'est pas l'inverse ?
		if (empty($conf->global->MAIN_PRODUCT_PERENTITY_SHARED)) {
			$product = new Product($this->db);
			$product->fetch($product_id);
			$productaccountancycode->entity = $product->entity > 0 ? $product->entity : $conf->entity;
		} else {
			$productaccountancycode->entity = $conf->entity;
		}

		$productaccountancycode->vat_code = $vat_code;
		$productaccountancycode->tva_tx = $vat_rate;
		$productaccountancycode->key_values = $key_values;

		$productaccountancycode->accountancy_code_sell = $accountancy_code_sell;
		$productaccountancycode->accountancy_code_sell_intra = $accountancy_code_sell_intra;
		$productaccountancycode->accountancy_code_sell_export = $accountancy_code_sell_export;
		$productaccountancycode->accountancy_code_buy = $accountancy_code_buy;
		$productaccountancycode->accountancy_code_buy_intra = $accountancy_code_buy_intra;
		$productaccountancycode->accountancy_code_buy_export = $accountancy_code_buy_export;

		$result = $productaccountancycode->create($user, $notrigger);
		if ($result > 0) {
			return $productaccountancycode->id;
		} else {
			$this->error = $productaccountancycode->error;
			$this->errors = $productaccountancycode->errors;
			return -1;
		}
	}

	/**
	 * Add the accountancy code for a vat rate and the other key value
	 *
	 * @param	int				$id								ID of the product accountancy code
	 * @param	User			$user							User to set accountancy code
	 * @param	int				$product_id						Product ID
     * @param	int				$product_type					If product ID not provided (10: all, 0: product, 1: service)
	 * @param	int				$free_line					Only free line
	 * @param	string			$vat_code						VAT code
	 * @param	int				$vat_rate						VAT rate
	 * @param	array			$key_values						List of Key value  array(field_name => value, ...)
	 * @param	string			$accountancy_code_sell			Accountancy code sell
	 * @param	string			$accountancy_code_sell_intra	Accountancy code sell intra
	 * @param	string			$accountancy_code_sell_export	Accountancy code sell export
	 * @param	string			$accountancy_code_buy			Accountancy code buy
	 * @param	string			$accountancy_code_buy_intra		Accountancy code buy intra
	 * @param	string			$accountancy_code_buy_export	Accountancy code buy export
	 * @param	int				$notrigger						Do not execute trigger
	 * @return	int												<0 if KO, >0 if OK
	 */
	public function updateAccountancyCode($id, User $user, $product_id, $product_type, $free_line, $vat_code, $vat_rate, $key_values = array(), $accountancy_code_sell = '', $accountancy_code_sell_intra = '', $accountancy_code_sell_export = '', $accountancy_code_buy = '', $accountancy_code_buy_intra = '', $accountancy_code_buy_export = '', $notrigger = 0)
	{
		dol_syslog(__METHOD__ . " userid=$user->id, lineid=$id, productid=$product_id, product_type=$product_type, free_line=$free_line, vat_code=$vat_code, vat_rate=$vat_rate" . ", key_values=" . json_encode($key_values) . ", accountancy_code_sell=$accountancy_code_sell, accountancy_code_sell_intra=$accountancy_code_sell_intra, accountancy_code_sell_export=$accountancy_code_sell_export, accountancy_code_buy=$accountancy_code_buy, accountancy_code_buy_intra=$accountancy_code_buy_intra, accountancy_code_buy_export=$accountancy_code_buy_export, notrigger=$notrigger");

		// Add accountancy code
		$productaccountancycode = new ProductAccountancyCode($this->db);

		$result = $productaccountancycode->fetch($id);
		if ($result > 0) {
			$productaccountancycode->oldcopy = clone $productaccountancycode;

			$productaccountancycode->context = $this->context;

			$productaccountancycode->fk_product = $product_id;
            $productaccountancycode->product_type = $product_type;
			$productaccountancycode->free_line = $free_line;
			$productaccountancycode->vat_code = $vat_code;
			$productaccountancycode->tva_tx = $vat_rate;
			$productaccountancycode->key_values = $key_values;

			$productaccountancycode->accountancy_code_sell = $accountancy_code_sell;
			$productaccountancycode->accountancy_code_sell_intra = $accountancy_code_sell_intra;
			$productaccountancycode->accountancy_code_sell_export = $accountancy_code_sell_export;
			$productaccountancycode->accountancy_code_buy = $accountancy_code_buy;
			$productaccountancycode->accountancy_code_buy_intra = $accountancy_code_buy_intra;
			$productaccountancycode->accountancy_code_buy_export = $accountancy_code_buy_export;

			$result = $productaccountancycode->update($id, $user, $notrigger);
		}

		if ($result < 0) {
			$this->error = $productaccountancycode->error;
			$this->errors = $productaccountancycode->errors;
			return -1;
		}
		return 1;
	}

	/**
	 * Add the accountancy code for a vat rate and the other key value
	 *
	 * @param	User			$user							User to set accountancy code
	 * @param	int				$id								ID of the product accountancy code
	 * @param	int				$notrigger						Do not execute trigger
	 * @return	int												<0 if KO, >0 if OK
	 */
	public function deleteAccountancyCode(User $user, $id, $notrigger = 0)
	{
		dol_syslog(__METHOD__ . " userid=$user->id, lineid=$id, notrigger=$notrigger");

		// Add accountancy code
		$productaccountancycode = new ProductAccountancyCode($this->db);

		$result = $productaccountancycode->fetch($id);
		if ($result > 0) {
			$result = $productaccountancycode->delete($user, $notrigger);
		}

		if ($result < 0) {
			$this->error = $productaccountancycode->error;
			$this->errors = $productaccountancycode->errors;
			return -1;
		}

		return 1;
	}

	/**
	 * Get the label of the provided accountancy code
	 *
	 * @param	string		$accountancy_code		Accountancy code
	 * @return	string
	 */
	public function getAccountancyCodeLabel($accountancy_code)
	{
		global $conf;

		if (!isset(self::$accountancy_code_label_cached[$accountancy_code])) {
			$label = $accountancy_code;

			if (!empty($conf->accounting->enabled)) {
				if (!empty($accountancy_code)) {
					require_once DOL_DOCUMENT_ROOT . '/accountancy/class/accountingaccount.class.php';
					$accountingaccount = new AccountingAccount($this->db);
					$accountingaccount->fetch('', $accountancy_code, 1);

					$label = $accountingaccount->getNomUrl(0, 1, 1, '', 1);
				}
			}

			self::$accountancy_code_label_cached[$accountancy_code] = $label;
		}

		return self::$accountancy_code_label_cached[$accountancy_code];
	}
}
