<?php
/*  Copyright (C) 2019      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	\file       htdocs/extendedproductaccountancy/core/triggers/interface_99_modExtendedProductAccountancy_ExtendedProductAccountancy.class.php
 *  \ingroup    extendedproductaccountancy
 *	\brief      File of class of triggers for extendedproductaccountancy module
 */


require_once DOL_DOCUMENT_ROOT.'/core/triggers/dolibarrtriggers.class.php';


/**
 *  Class of triggers for ExtendedProductAccountancy module
 */
class InterfaceExtendedProductAccountancy extends DolibarrTriggers
{
	public $family = 'extendedproductaccountancy';
	public $description = "Triggers of this module catch triggers event for ExtendedProductAccountancy module.";
	public $version = self::VERSION_DOLIBARR;
	public $picto = 'technic';


	/**
	 * Function called when a Dolibarrr business event is done.
	 * All functions "runTrigger" are triggered if file is inside directory htdocs/core/triggers or htdocs/module/code/triggers (and declared)
	 *
	 * @param string		$action		Event action code
	 * @param Object		$object     Object
	 * @param User		    $user       Object user
	 * @param Translate 	$langs      Object langs
	 * @param conf		    $conf       Object conf
	 * @return int         				<0 if KO, 0 if no triggered ran, >0 if OK
	 */
	public function runTrigger($action, $object, User $user, Translate $langs, Conf $conf)
	{
		if (empty($conf->extendedproductaccountancy->enabled)) return 0;     // Module not active, we do nothing

		switch ($action) {
			// Products
			case 'PRODUCT_DELETE':
				// Delete product accountancy code table records
				$sql = "DELETE FROM " . MAIN_DB_PREFIX . 'product_accountancy_code';
				$sql .= " WHERE fk_product = " . (int)$object->id;

				$result = $this->db->query($sql);
				if (!$result) {
					$this->errors[] = $this->db->lasterror();
					return -1;
				}

				dol_syslog("Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->id);
				break;
		}

		return 0;
	}
}